@@ -104,8 +104,8 @@
  *   <varlistentry>
  *     <term>GST_MESSAGE_CLOCK_PROVIDE</term>
  *     <listitem><para> This message is generated when an element
- *     can provide a clock. This mostly happens when a new clock 
- *     provider is added to the bin. The default behaviour of the bin is to 
+ *     can provide a clock. This mostly happens when a new clock
+ *     provider is added to the bin. The default behaviour of the bin is to
  *     mark the currently selected clock as dirty, which will perform a clock
  *     recalculation the next time the bin is asked to provide a clock.
  *     This message is never sent tot the application but is forwarded to
@@ -128,7 +128,7 @@
  *     and the bin is a toplevel bin (ie. has no parent),
  *     use the cached previous value. If no previous value was cached, the
  *     query is sent to all sink elements in the bin and the MAXIMUM of all
- *     values is returned. If the bin is a toplevel bin the value is cached. 
+ *     values is returned. If the bin is a toplevel bin the value is cached.
  *     If no sinks are available in the bin, the query fails.
  *     </para></listitem>
  *   </varlistentry>
@@ -171,11 +171,17 @@
 #include "gstutils.h"
 #include "gstchildproxy.h"
 
-/* enable for DURATION caching. 
+/* enable for DURATION caching.
  * FIXME currently too many elements don't update
  * their duration when it changes so we return inaccurate values. */
 #undef DURATION_CACHING
 
+/* latency is by default enabled now.
+ * live-preroll and no-live-preroll in the environment var GST_COMPAT
+ * to enables or disable it respectively.
+ */
+static gboolean enable_latency = TRUE;
+
 GST_DEBUG_CATEGORY_STATIC (bin_debug);
 #define GST_CAT_DEFAULT bin_debug
 
@@ -185,13 +191,24 @@ GST_ELEMENT_DETAILS ("Generic bin",
     "Simple container object",
     "Erik Walthinsen <omega@cse.ogi.edu>," "Wim Taymans <wim@fluendo.com>");
 
+struct _GstBinPrivate
+{
+  gboolean asynchandling;
+};
+
 static void gst_bin_dispose (GObject * object);
 
-static void gst_bin_recalc_state (GstBin * bin, gboolean force);
+static void gst_bin_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+static void gst_bin_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
+
 static GstStateChangeReturn gst_bin_change_state_func (GstElement * element,
     GstStateChange transition);
 static GstStateChangeReturn gst_bin_get_state_func (GstElement * element,
     GstState * state, GstState * pending, GstClockTime timeout);
+static void bin_handle_async_done (GstBin * bin, GstMessage ** message);
+static void bin_push_state_continue (GstBin * bin);
 
 static gboolean gst_bin_add_func (GstBin * bin, GstElement * element);
 static gboolean gst_bin_remove_func (GstBin * bin, GstElement * element);
@@ -215,7 +232,7 @@ static void gst_bin_restore_thyself (GstObject * object, xmlNodePtr self);
 
 static void bin_remove_messages (GstBin * bin, GstObject * src,
     GstMessageType types);
-static void gst_bin_recalc_func (GstBin * child, gpointer data);
+static void gst_bin_continue_func (GstBin * child, gpointer data);
 static gint bin_element_is_sink (GstElement * child, GstBin * bin);
 static gint bin_element_is_src (GstElement * child, GstBin * bin);
 
@@ -231,7 +248,8 @@ enum
 
 enum
 {
-  PROP_0
+  PROP_0,
+  PROP_ASYNC_HANDLING
       /* FILL ME */
 };
 
@@ -252,6 +270,7 @@ GType
 gst_bin_get_type (void)
 {
   static GType gst_bin_type = 0;
+  const gchar *compat;
 
   if (G_UNLIKELY (gst_bin_type == 0)) {
     static const GTypeInfo bin_info = {
@@ -280,6 +299,15 @@ gst_bin_get_type (void)
 
     GST_DEBUG_CATEGORY_INIT (bin_debug, "bin", GST_DEBUG_BOLD,
         "debugging info for the 'bin' container element");
+
+    /* compatibility stuff */
+    compat = g_getenv ("GST_COMPAT");
+    if (compat != NULL) {
+      if (strstr (compat, "no-live-preroll"))
+        enable_latency = FALSE;
+      else if (strstr (compat, "live-preroll"))
+        enable_latency = TRUE;
+    }
   }
   return gst_bin_type;
 }
@@ -347,6 +375,23 @@ gst_bin_class_init (GstBinClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
+  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_bin_set_property);
+  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_bin_get_property);
+
+  /**
+   * GstBin:async-handling
+   *
+   * If set to #TRUE, the bin will handle asynchronous state changes.
+   * This should be used only if the bin subclass is modifying the state
+   * of its childs on its own.
+   *
+   * Since: 0.10.13
+   */
+  g_object_class_install_property (gobject_class, PROP_ASYNC_HANDLING,
+      g_param_spec_boolean ("async-handling", "Async Handling",
+          "The bin will handle Asynchronous state changes",
+          FALSE, G_PARAM_READWRITE));
+
   /**
    * GstBin::element-added:
    * @bin: the #GstBin
@@ -398,7 +443,7 @@ gst_bin_class_init (GstBinClass * klass)
   GST_DEBUG ("creating bin thread pool");
   err = NULL;
   klass->pool =
-      g_thread_pool_new ((GFunc) gst_bin_recalc_func, NULL, -1, FALSE, &err);
+      g_thread_pool_new ((GFunc) gst_bin_continue_func, NULL, -1, FALSE, &err);
   if (err != NULL) {
     g_critical ("could alloc threadpool %s", err->message);
   }
@@ -413,9 +458,8 @@ gst_bin_init (GstBin * bin)
   bin->children = NULL;
   bin->children_cookie = 0;
   bin->messages = NULL;
-  bin->polling = FALSE;
-  bin->state_dirty = FALSE;
   bin->provided_clock = NULL;
+  bin->state_dirty = FALSE;
   bin->clock_dirty = FALSE;
 
   /* Set up a bus for listening to child elements */
@@ -424,6 +468,9 @@ gst_bin_init (GstBin * bin)
   GST_DEBUG_OBJECT (bin, "using bus %" GST_PTR_FORMAT " to listen to children",
       bus);
   gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bin_bus_handler, bin);
+
+  bin->priv = g_new0 (GstBinPrivate, 1);
+  bin->priv->asynchandling = FALSE;
 }
 
 static void
@@ -450,6 +497,11 @@ gst_bin_dispose (GObject * object)
         GST_STR_NULL (GST_OBJECT_NAME (object)));
   }
 
+  if (bin->priv) {
+    g_free (bin->priv);
+    bin->priv = NULL;
+  }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -467,6 +519,46 @@ gst_bin_new (const gchar * name)
   return gst_element_factory_make ("bin", name);
 }
 
+static void
+gst_bin_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstBin *gstbin;
+
+  gstbin = GST_BIN (object);
+
+  switch (prop_id) {
+    case PROP_ASYNC_HANDLING:
+      GST_OBJECT_LOCK (gstbin);
+      gstbin->priv->asynchandling = g_value_get_boolean (value);
+      GST_OBJECT_UNLOCK (gstbin);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_bin_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstBin *gstbin;
+
+  gstbin = GST_BIN (object);
+
+  switch (prop_id) {
+    case PROP_ASYNC_HANDLING:
+      GST_OBJECT_LOCK (gstbin);
+      g_value_set_boolean (value, gstbin->priv->asynchandling);
+      GST_OBJECT_UNLOCK (gstbin);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 /* set the index on all elements in this bin
  *
  * MT safe
@@ -520,7 +612,7 @@ gst_bin_set_clock_func (GstElement * element, GstClock * clock)
  *
  * The ref of the returned clock in increased so unref after usage.
  *
- * We loop the elements in state order and pick the last clock we can 
+ * We loop the elements in state order and pick the last clock we can
  * get. This makes sure we get a clock from the source.
  *
  * MT safe
@@ -617,7 +709,7 @@ message_check (GstMessage * message, MessageFind * target)
 }
 
 /* with LOCK, returns TRUE if message had a valid SRC, takes ref on
- * the message. 
+ * the message.
  *
  * A message that is cached and has the same SRC and type is replaced
  * by the given message.
@@ -680,12 +772,13 @@ bin_remove_messages (GstBin * bin, GstObject * src, GstMessageType types)
 
     if (message_check (message, &find) == 0) {
       GST_DEBUG_OBJECT (GST_MESSAGE_SRC (message),
-          "deleting message of types %d", types);
+          "deleting message %p of types 0x%08x", message, types);
       bin->messages = g_list_delete_link (bin->messages, walk);
       gst_message_unref (message);
     } else {
       GST_DEBUG_OBJECT (GST_MESSAGE_SRC (message),
-          "not deleting message of type %d", GST_MESSAGE_TYPE (message));
+          "not deleting message %p of type 0x%08x", message,
+          GST_MESSAGE_TYPE (message));
     }
   }
 }
@@ -803,6 +896,7 @@ gst_bin_add_func (GstBin * bin, GstElement * element)
    * that is not important right now. When the pipeline goes to PLAYING,
    * a new clock will be selected */
   gst_element_set_clock (element, GST_ELEMENT_CLOCK (bin));
+  GST_DEBUG_OBJECT (bin, "marking state dirty");
   bin->state_dirty = TRUE;
   GST_OBJECT_UNLOCK (bin);
 
@@ -848,7 +942,6 @@ had_parent:
   }
 }
 
-
 /**
  * gst_bin_add:
  * @bin: a #GstBin
@@ -880,7 +973,8 @@ gst_bin_add (GstBin * bin, GstElement * element)
     goto no_function;
 
   GST_CAT_DEBUG (GST_CAT_PARENTAGE, "adding element %s to bin %s",
-      GST_ELEMENT_NAME (element), GST_ELEMENT_NAME (bin));
+      GST_STR_NULL (GST_ELEMENT_NAME (element)),
+      GST_STR_NULL (GST_ELEMENT_NAME (bin)));
 
   result = bclass->add_element (bin, element);
 
@@ -905,7 +999,11 @@ gst_bin_remove_func (GstBin * bin, GstElement * element)
   gchar *elem_name;
   GstIterator *it;
   gboolean is_sink;
-  GstMessage *clock_message = NULL;
+  GstMessage *clock_message = NULL, *async_message = NULL, *smessage = NULL;
+  GList *walk, *next;
+  gboolean other_async = FALSE, this_async = FALSE, cont = FALSE;
+
+  GST_DEBUG_OBJECT (bin, "element :%s", GST_ELEMENT_NAME (element));
 
   GST_OBJECT_LOCK (element);
   /* Check if the element is already being removed and immediately
@@ -957,12 +1055,58 @@ gst_bin_remove_func (GstBin * bin, GstElement * element)
         gst_message_new_clock_lost (GST_OBJECT_CAST (bin), bin->provided_clock);
   }
 
+  /* remove messages for the element, if there was a pending ASYNC_START
+   * message we must see if removing the element caused the bin to lose its
+   * async state. */
+  for (walk = bin->messages; walk; walk = next) {
+    GstMessage *message = (GstMessage *) walk->data;
+    GstElement *src = GST_ELEMENT_CAST (GST_MESSAGE_SRC (message));
+
+    next = g_list_next (walk);
+
+    if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ASYNC_START) {
+      if (src == element)
+        this_async = TRUE;
+      else
+        other_async = TRUE;
+
+      GST_DEBUG_OBJECT (GST_MESSAGE_SRC (message),
+          "looking at message %p", message);
+    }
+    if (src == element) {
+      /* delete all message types */
+      GST_DEBUG_OBJECT (GST_MESSAGE_SRC (message),
+          "deleting message %p of element \"%s\"", message, elem_name);
+      bin->messages = g_list_delete_link (bin->messages, walk);
+      gst_message_unref (message);
+    }
+  }
+  /* all other elements were not async and we removed the async one,
+   * post a ASYNC_DONE message because we are not async anymore now. */
+  if (!other_async && this_async) {
+    GST_DEBUG_OBJECT (bin, "we removed the last async element");
+
+    cont = ((GST_OBJECT_PARENT (bin) == NULL) || bin->priv->asynchandling);
+    if (!cont) {
+      bin_handle_async_done (bin, &smessage);
+      async_message = gst_message_new_async_done (GST_OBJECT_CAST (bin));
+    }
+  }
+  GST_DEBUG_OBJECT (bin, "marking state dirty");
   bin->state_dirty = TRUE;
   GST_OBJECT_UNLOCK (bin);
 
-  if (clock_message) {
+  if (clock_message)
     gst_element_post_message (GST_ELEMENT_CAST (bin), clock_message);
-  }
+
+  if (smessage)
+    gst_element_post_message (GST_ELEMENT_CAST (bin), smessage);
+
+  if (async_message)
+    gst_element_post_message (GST_ELEMENT_CAST (bin), async_message);
+
+  if (cont)
+    bin_push_state_continue (bin);
 
   GST_CAT_INFO_OBJECT (GST_CAT_PARENTAGE, bin, "removed child \"%s\"",
       elem_name);
@@ -1251,7 +1395,7 @@ src_iterator_filter (GstElement * child, GstBin * bin)
  * gst_bin_iterate_sources:
  * @bin: a #GstBin
  *
- * Gets an iterator for all elements in the bin that have no sinkpads and have 
+ * Gets an iterator for all elements in the bin that have no sinkpads and have
  * the #GST_ELEMENT_IS_SINK flag unset.
  *
  * Each element yielded by the iterator will have its refcount increased, so
@@ -1276,26 +1420,6 @@ gst_bin_iterate_sources (GstBin * bin)
   return result;
 }
 
-/*
- * MT safe
- */
-static GstStateChangeReturn
-gst_bin_get_state_func (GstElement * element, GstState * state,
-    GstState * pending, GstClockTime timeout)
-{
-  GstBin *bin = GST_BIN (element);
-  GstStateChangeReturn ret;
-
-  GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "getting state");
-
-  /* do a non forced recalculation of the state */
-  gst_bin_recalc_state (bin, FALSE);
-
-  ret = parent_class->get_state (element, state, pending, timeout);
-
-  return ret;
-}
-
 static void
 gst_bin_recalc_state (GstBin * bin, gboolean force)
 {
@@ -1403,6 +1527,7 @@ restart:
 
 done:
   bin->polling = FALSE;
+  GST_STATE_RETURN (bin) = ret;
   GST_OBJECT_UNLOCK (bin);
 
   /* now we can take the state lock, it is possible that new elements
@@ -1460,6 +1585,28 @@ unknown_state:
   }
 }
 
+/*
+ * MT safe
+ */
+static GstStateChangeReturn
+gst_bin_get_state_func (GstElement * element, GstState * state,
+    GstState * pending, GstClockTime timeout)
+{
+  GstBin *bin = GST_BIN (element);
+  GstStateChangeReturn ret;
+
+  GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "getting state");
+
+  /* do a non forced recalculation of the state */
+  GST_STATE_LOCK (bin);
+  gst_bin_recalc_state (bin, FALSE);
+  GST_STATE_UNLOCK (bin);
+
+  ret = parent_class->get_state (element, state, pending, timeout);
+
+  return ret;
+}
+
 /***********************************************
  * Topologically sorted iterator
  * see http://en.wikipedia.org/wiki/Topological_sorting
@@ -1745,10 +1892,13 @@ gst_bin_iterate_sorted (GstBin * bin)
 }
 
 static GstStateChangeReturn
-gst_bin_element_set_state (GstBin * bin, GstElement * element, GstState pending)
+gst_bin_element_set_state (GstBin * bin, GstElement * element,
+    GstClockTime base_time, GstState current, GstState next)
 {
   GstStateChangeReturn ret;
   gboolean locked;
+  GList *found;
+  MessageFind find;
 
   /* peel off the locked flag */
   GST_OBJECT_LOCK (element);
@@ -1756,18 +1906,66 @@ gst_bin_element_set_state (GstBin * bin, GstElement * element, GstState pending)
   GST_OBJECT_UNLOCK (element);
 
   /* skip locked elements */
-  if (G_UNLIKELY (locked)) {
-    GST_DEBUG_OBJECT (element,
-        "element is locked, pretending state change succeeded");
-    ret = GST_STATE_CHANGE_SUCCESS;
-    goto done;
+  if (G_UNLIKELY (locked))
+    goto locked;
+
+  /* the element was busy with an upwards async state change, we must wait for
+   * an ASYNC_DONE message before we attemp to change the state. */
+  find.src = GST_OBJECT_CAST (element);
+  find.types = GST_MESSAGE_ASYNC_START;
+
+  GST_OBJECT_LOCK (bin);
+  if ((found = g_list_find_custom (bin->messages, &find,
+              (GCompareFunc) message_check))) {
+#ifndef GST_DISABLE_GST_DEBUG
+    GstMessage *message = GST_MESSAGE_CAST (found->data);
+#endif
+
+    GST_DEBUG_OBJECT (element, "element message %p, %s async busy",
+        message, GST_ELEMENT_NAME (GST_MESSAGE_SRC (message)));
+    /* only wait for upward state changes */
+    if (next > current) {
+      /* We found an async element check if we can force its state to change or
+       * if we have to wait for it to preroll. */
+      if (G_UNLIKELY (!enable_latency)) {
+        g_warning ("Future versions of GStreamer will wait for element \"%s\"\n"
+            "\tto preroll in order to perform correct latency calculations.\n"
+            "\tPlease verify that the application continues to work correctly by\n"
+            "\tsetting the environment variable GST_COMPAT to a value containing\n"
+            "\tthe string 'live-preroll'.", GST_ELEMENT_NAME (element));
+        goto no_latency;
+      }
+      goto was_busy;
+    }
   }
+no_latency:
+  GST_OBJECT_UNLOCK (bin);
+
+  GST_DEBUG_OBJECT (bin,
+      "setting element %s to %s, base_time %" GST_TIME_FORMAT,
+      GST_ELEMENT_NAME (element), gst_element_state_get_name (next),
+      GST_TIME_ARGS (base_time));
+
+  /* set base_time on child */
+  gst_element_set_base_time (element, base_time);
 
   /* change state */
-  ret = gst_element_set_state (element, pending);
+  ret = gst_element_set_state (element, next);
 
-done:
   return ret;
+
+locked:
+  {
+    GST_DEBUG_OBJECT (element,
+        "element is locked, pretending state change succeeded");
+    return GST_STATE_CHANGE_SUCCESS;
+  }
+was_busy:
+  {
+    GST_DEBUG_OBJECT (element, "element is was busy delaying state change");
+    GST_OBJECT_UNLOCK (bin);
+    return GST_STATE_CHANGE_ASYNC;
+  }
 }
 
 /* gst_iterator_fold functions for pads_activate
@@ -1813,7 +2011,7 @@ iterator_activate_fold_with_resync (GstIterator * iter, gpointer user_data)
         /* all pads iterated, return collected value */
         goto done;
       default:
-        /* iterator returned _ERROR or premature end with _OK, 
+        /* iterator returned _ERROR or premature end with _OK,
          * mark an error and exit */
         g_value_set_boolean (&ret, FALSE);
         goto done;
@@ -1924,11 +2122,8 @@ restart:
 
         child = GST_ELEMENT_CAST (data);
 
-        /* set base_time on child */
-        gst_element_set_base_time (child, base_time);
-
-        /* set state now */
-        ret = gst_bin_element_set_state (bin, child, next);
+        /* set state and base_time now */
+        ret = gst_bin_element_set_state (bin, child, base_time, current, next);
 
         switch (ret) {
           case GST_STATE_CHANGE_SUCCESS:
@@ -1938,11 +2133,13 @@ restart:
                 gst_element_state_get_name (next));
             break;
           case GST_STATE_CHANGE_ASYNC:
+          {
             GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
                 "child '%s' is changing state asynchronously to %s",
                 GST_ELEMENT_NAME (child), gst_element_state_get_name (next));
             have_async = TRUE;
             break;
+          }
           case GST_STATE_CHANGE_FAILURE:
             GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
                 "child '%s' failed to go to state %d(%s)",
@@ -1990,10 +2187,11 @@ done:
   gst_iterator_free (it);
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
-      "done changing bin's state from %s to %s, now in %s, ret %d",
+      "done changing bin's state from %s to %s, now in %s, ret %s",
       gst_element_state_get_name (current),
       gst_element_state_get_name (next),
-      gst_element_state_get_name (GST_STATE (element)), ret);
+      gst_element_state_get_name (GST_STATE (element)),
+      gst_element_state_change_return_get_name (ret));
 
   return ret;
 
@@ -2066,20 +2264,144 @@ gst_bin_send_event (GstElement * element, GstEvent * event)
 }
 
 static void
-gst_bin_recalc_func (GstBin * bin, gpointer data)
+gst_bin_continue_func (GstBin * bin, gpointer data)
 {
-  GST_DEBUG_OBJECT (bin, "doing state recalc");
+  GstState current, next, pending, target, old_state, old_next;
+  GstStateChangeReturn old_ret, ret;
+  GstStateChange transition;
+  gboolean post;
+
+  GST_DEBUG_OBJECT (bin, "waiting for state lock");
   GST_STATE_LOCK (bin);
-  gst_bin_recalc_state (bin, FALSE);
+
+  GST_DEBUG_OBJECT (bin, "doing state continue");
+  GST_OBJECT_LOCK (bin);
+
+  old_ret = GST_STATE_RETURN (bin);
+  GST_STATE_RETURN (bin) = GST_STATE_CHANGE_SUCCESS;
+  target = GST_STATE_TARGET (bin);
+  pending = GST_STATE_PENDING (bin);
+
+  /* check if there is something to commit */
+  if (pending == GST_STATE_VOID_PENDING)
+    goto nothing_pending;
+
+  old_state = GST_STATE (bin);
+  /* this is the state we should go to next */
+  old_next = GST_STATE_NEXT (bin);
+
+  GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "target %s",
+      gst_element_state_get_name (target));
+
+  if (old_next == GST_STATE_PLAYING) {
+    post = FALSE;
+  } else {
+    post = TRUE;
+  }
+
+  /* we're going to PLAYING, always do the PAUSED->PLAYING state change */
+  if (target == GST_STATE_PLAYING) {
+    next = GST_STATE_PAUSED;
+    GST_STATE_PENDING (bin) = pending = target;
+  } else {
+    next = old_next;
+  }
+
+  /* update current state */
+  current = GST_STATE (bin) = next;
+
+  /* see if we reached the final state */
+  if (pending == current)
+    goto complete;
+
+  next = GST_STATE_GET_NEXT (current, pending);
+  transition = (GstStateChange) GST_STATE_TRANSITION (current, next);
+
+  GST_STATE_NEXT (bin) = next;
+  /* mark busy */
+  GST_STATE_RETURN (bin) = GST_STATE_CHANGE_ASYNC;
+  GST_OBJECT_UNLOCK (bin);
+
+  if (post) {
+    GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin,
+        "committing state from %s to %s, pending %s",
+        gst_element_state_get_name (old_state),
+        gst_element_state_get_name (old_next),
+        gst_element_state_get_name (pending));
+
+    gst_element_post_message (GST_ELEMENT_CAST (bin),
+        gst_message_new_state_changed (GST_OBJECT_CAST (bin),
+            old_state, old_next, pending));
+  }
+
+  GST_DEBUG_OBJECT (bin, "posting ASYNC_DONE");
+  gst_element_post_message (GST_ELEMENT_CAST (bin),
+      gst_message_new_async_done (GST_OBJECT_CAST (bin)));
+
+  GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin,
+      "continue state change %s to %s, final %s",
+      gst_element_state_get_name (current),
+      gst_element_state_get_name (next), gst_element_state_get_name (pending));
+
+  ret = gst_element_change_state (GST_ELEMENT_CAST (bin), transition);
+
+done:
   GST_STATE_UNLOCK (bin);
-  GST_DEBUG_OBJECT (bin, "state recalc done");
+  GST_DEBUG_OBJECT (bin, "state continue done");
   gst_object_unref (bin);
+
+  return;
+
+nothing_pending:
+  {
+    GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "nothing pending");
+    GST_OBJECT_UNLOCK (bin);
+
+    GST_DEBUG_OBJECT (bin, "posting ASYNC_DONE");
+    gst_element_post_message (GST_ELEMENT_CAST (bin),
+        gst_message_new_async_done (GST_OBJECT_CAST (bin)));
+
+    goto done;
+  }
+complete:
+  {
+    GST_STATE_PENDING (bin) = GST_STATE_VOID_PENDING;
+    GST_STATE_NEXT (bin) = GST_STATE_VOID_PENDING;
+
+    GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "completed state change to %s",
+        gst_element_state_get_name (pending));
+    GST_OBJECT_UNLOCK (bin);
+
+    /* don't post silly messages with the same state. This can happen
+     * when an element state is changed to what it already was. For bins
+     * this can be the result of a lost state, which we check with the
+     * previous return value.
+     * We do signal the cond though as a _get_state() might be blocking
+     * on it. */
+    if (old_state != old_next || old_ret == GST_STATE_CHANGE_ASYNC) {
+      GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin,
+          "changed %s to %s, VOID pending",
+          gst_element_state_get_name (old_state),
+          gst_element_state_get_name (old_next));
+
+      gst_element_post_message (GST_ELEMENT_CAST (bin),
+          gst_message_new_state_changed (GST_OBJECT_CAST (bin),
+              old_state, old_next, GST_STATE_VOID_PENDING));
+    }
+
+    GST_DEBUG_OBJECT (bin, "posting ASYNC_DONE");
+    gst_element_post_message (GST_ELEMENT_CAST (bin),
+        gst_message_new_async_done (GST_OBJECT_CAST (bin)));
+
+    GST_STATE_BROADCAST (bin);
+
+    goto done;
+  }
 }
 
 static GstBusSyncReply
 bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
 {
-
   GstBinClass *bclass;
 
   bclass = GST_BIN_GET_CLASS (bin);
@@ -2091,10 +2413,150 @@ bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
   return GST_BUS_DROP;
 }
 
+static void
+bin_push_state_continue (GstBin * bin)
+{
+  GstBinClass *klass;
+
+  /* mark the bin dirty */
+  GST_OBJECT_LOCK (bin);
+  klass = GST_BIN_GET_CLASS (bin);
+  GST_DEBUG_OBJECT (bin, "pushing continue on thread pool");
+  gst_object_ref (bin);
+  g_thread_pool_push (klass->pool, bin, NULL);
+  GST_OBJECT_UNLOCK (bin);
+}
+
+/* an element started an async state change, if we were not busy with a state
+ * change, we perform a lost state.
+ */
+static void
+bin_handle_async_start (GstBin * bin, GstMessage ** message)
+{
+  GstState old_state, new_state;
+
+  if (GST_STATE_RETURN (bin) == GST_STATE_CHANGE_FAILURE)
+    goto had_error;
+
+  if (GST_STATE_PENDING (bin) != GST_STATE_VOID_PENDING)
+    goto was_busy;
+
+  old_state = GST_STATE (bin);
+
+  if (old_state > GST_STATE_PAUSED)
+    new_state = GST_STATE_PAUSED;
+  else
+    new_state = old_state;
+
+  GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, bin,
+      "lost state of %s, new %s", gst_element_state_get_name (old_state),
+      gst_element_state_get_name (new_state));
+
+  GST_STATE (bin) = new_state;
+  GST_STATE_NEXT (bin) = new_state;
+  GST_STATE_PENDING (bin) = new_state;
+  GST_STATE_RETURN (bin) = GST_STATE_CHANGE_ASYNC;
+
+  *message = gst_message_new_state_changed (GST_OBJECT_CAST (bin),
+      new_state, new_state, new_state);
+
+  return;
+
+had_error:
+  {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, bin, "we had an error");
+    return;
+  }
+was_busy:
+  {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, bin, "state change busy");
+    return;
+  }
+}
+
+static void
+bin_handle_async_done (GstBin * bin, GstMessage ** message)
+{
+  GstState pending;
+  GstStateChangeReturn old_ret;
+  GstState old_state, old_next;
+  GstState current, next;
+  gboolean cont;
+
+  old_ret = GST_STATE_RETURN (bin);
+  GST_STATE_RETURN (bin) = GST_STATE_CHANGE_SUCCESS;
+  pending = GST_STATE_PENDING (bin);
+
+  /* check if there is something to commit */
+  if (pending == GST_STATE_VOID_PENDING)
+    goto nothing_pending;
+
+  old_state = GST_STATE (bin);
+  /* this is the state we should go to next */
+  old_next = GST_STATE_NEXT (bin);
+  /* update current state */
+  current = GST_STATE (bin) = old_next;
+
+  /* see if we need to continue the state change on our own. This happens when
+   * we were asked to do so or when we are the toplevel bin. */
+  cont = bin->priv->asynchandling || (GST_OBJECT_PARENT (bin) == NULL);
+
+  /* see if we reached the final state. If we are not a toplevel bin we also
+   * must stop at this state change, the parent will set us to the required
+   * state eventually. */
+  if (pending == current || !cont)
+    goto complete;
+
+  next = GST_STATE_GET_NEXT (current, pending);
+
+  GST_STATE_NEXT (bin) = next;
+  /* mark busy */
+  GST_STATE_RETURN (bin) = GST_STATE_CHANGE_ASYNC;
+
+  GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin,
+      "committing state from %s to %s, pending %s",
+      gst_element_state_get_name (old_state),
+      gst_element_state_get_name (old_next),
+      gst_element_state_get_name (pending));
+
+  *message = gst_message_new_state_changed (GST_OBJECT_CAST (bin),
+      old_state, old_next, pending);
+
+  return;
+
+nothing_pending:
+  {
+    GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "nothing pending");
+    return;
+  }
+complete:
+  {
+    GST_STATE_PENDING (bin) = GST_STATE_VOID_PENDING;
+    GST_STATE_NEXT (bin) = GST_STATE_VOID_PENDING;
+
+    GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "completed state change");
+
+    /* don't post silly messages with the same state. This can happen
+     * when an element state is changed to what it already was. For bins
+     * this can be the result of a lost state, which we check with the
+     * previous return value.
+     * We do signal the cond though as a _get_state() might be blocking
+     * on it. */
+    if (old_state != old_next || old_ret == GST_STATE_CHANGE_ASYNC) {
+      *message = gst_message_new_state_changed (GST_OBJECT_CAST (bin),
+          old_state, old_next, GST_STATE_VOID_PENDING);
+    }
+
+    GST_STATE_BROADCAST (bin);
+
+    return;
+  }
+}
+
 /* handle child messages:
  *
  * This method is called synchronously when a child posts a message on
- * the internal bus. 
+ * the internal bus.
  *
  * GST_MESSAGE_EOS: This message is only posted by sinks
  *     in the PLAYING state. If all sinks posted the EOS message, post
@@ -2123,8 +2585,8 @@ bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
  * GST_MESSAGE_CLOCK_LOST: This message is posted by an element when it
  *     can no longer provide a clock. The default bin behaviour is to
  *     check if the lost clock was the one provided by the bin. If so and
- *     we are currently in the PLAYING state, we forward the message to 
- *     our parent. 
+ *     we are currently in the PLAYING state, we forward the message to
+ *     our parent.
  *     This message is also generated when we remove a clock provider from
  *     a bin. If this message is received by the application, it should
  *     PAUSE the pipeline and set it back to PLAYING to force a new clock
@@ -2138,15 +2600,29 @@ bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
  *     This message is never sent to the application but is forwarded to
  *     the parent.
  *
+ * GST_MESSAGE_ASYNC_START: Create an internal ELEMENT message that stores
+ *     the state of the element and the fact that the element will need a
+ *     new base_time.
+ *
+ * GST_MESSAGE_ASYNC_DONE: Find the internal ELEMENT message we kept for the
+ *     element when it posted ASYNC_START. If all elements are done, post a
+ *     ASYNC_DONE message to the parent.
+ *
  * OTHER: post upwards.
  */
 static void
 gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
 {
-  GST_DEBUG_OBJECT (bin, "[msg %p] handling child message of type %s",
-      message, GST_MESSAGE_TYPE_NAME (message));
+  GstObject *src;
+  GstMessageType type;
 
-  switch (GST_MESSAGE_TYPE (message)) {
+  src = GST_MESSAGE_SRC (message);
+  type = GST_MESSAGE_TYPE (message);
+
+  GST_DEBUG_OBJECT (bin, "[msg %p] handling child %s message of type %s",
+      message, GST_ELEMENT_NAME (src), GST_MESSAGE_TYPE_NAME (message));
+
+  switch (type) {
     case GST_MESSAGE_EOS:
     {
       gboolean eos;
@@ -2167,48 +2643,15 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
     }
     case GST_MESSAGE_STATE_DIRTY:
     {
-      GstObject *src;
-      GstBinClass *klass;
-
-      src = GST_MESSAGE_SRC (message);
-
-      GST_DEBUG_OBJECT (bin, "%s gave state dirty", GST_ELEMENT_NAME (src));
-
-      /* mark the bin dirty */
-      GST_OBJECT_LOCK (bin);
-      GST_DEBUG_OBJECT (bin, "marking dirty");
-      bin->state_dirty = TRUE;
-
-      if (GST_OBJECT_PARENT (bin))
-        goto not_toplevel;
+      GST_WARNING_OBJECT (bin, "received deprecated STATE_DIRTY message");
 
       /* free message */
       gst_message_unref (message);
-
-      klass = GST_BIN_GET_CLASS (bin);
-      if (!bin->polling) {
-        GST_DEBUG_OBJECT (bin, "pushing recalc on thread pool");
-        gst_object_ref (bin);
-        g_thread_pool_push (klass->pool, bin, NULL);
-      } else {
-        GST_DEBUG_OBJECT (bin,
-            "state recalc already in progress, not pushing new recalc");
-      }
-      GST_OBJECT_UNLOCK (bin);
       break;
-
-      /* non toplevel bins just forward the message and don't start
-       * a recalc themselves */
-    not_toplevel:
-      {
-        GST_OBJECT_UNLOCK (bin);
-        GST_DEBUG_OBJECT (bin, "not toplevel, forwarding");
-        goto forward;
-      }
     }
     case GST_MESSAGE_SEGMENT_START:
       GST_OBJECT_LOCK (bin);
-      /* replace any previous segment_start message from this source 
+      /* replace any previous segment_start message from this source
        * with the new segment start message */
       bin_replace_message (bin, message, GST_MESSAGE_SEGMENT_START);
       GST_OBJECT_UNLOCK (bin);
@@ -2267,7 +2710,7 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
 
       GST_OBJECT_LOCK (bin);
       bin->clock_dirty = TRUE;
-      /* if we lost the clock that we provided, post to parent but 
+      /* if we lost the clock that we provided, post to parent but
        * only if we are PLAYING. */
       provided = (clock == bin->provided_clock);
       playing = (GST_STATE (bin) == GST_STATE_PLAYING);
@@ -2310,6 +2753,118 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
       gst_message_unref (message);
       break;
     }
+    case GST_MESSAGE_ASYNC_START:
+    {
+      gboolean forward = FALSE, new_base_time;
+      GstState target;
+      GstMessage *smessage = NULL;
+
+      GST_DEBUG_OBJECT (bin, "ASYNC_START message %p, %s", message,
+          GST_OBJECT_NAME (src));
+
+      gst_message_parse_async_start (message, &new_base_time);
+
+      GST_OBJECT_LOCK (bin);
+      /* we ignore the message if we are going to <= READY */
+      target = GST_STATE_TARGET (bin);
+      if (target <= GST_STATE_READY)
+        goto ignore_start_message;
+
+      bin_replace_message (bin, message, GST_MESSAGE_ASYNC_START);
+
+      bin_handle_async_start (bin, &smessage);
+
+      /* prepare an ASYNC_START message */
+      if (GST_OBJECT_PARENT (bin) && (!bin->priv->asynchandling)) {
+        forward = TRUE;
+        message =
+            gst_message_new_async_start (GST_OBJECT_CAST (bin), new_base_time);
+      } else {
+        message = NULL;
+      }
+      GST_OBJECT_UNLOCK (bin);
+
+      if (smessage)
+        gst_element_post_message (GST_ELEMENT_CAST (bin), smessage);
+
+      if (forward)
+        goto forward;
+      else
+        break;
+
+    ignore_start_message:
+      {
+        GST_DEBUG_OBJECT (bin, "ignoring message, target %s",
+            gst_element_state_get_name (target));
+        GST_OBJECT_UNLOCK (bin);
+        gst_message_unref (message);
+        break;
+      }
+    }
+    case GST_MESSAGE_ASYNC_DONE:
+    {
+      gboolean done = FALSE, toplevel = FALSE;
+      GstState target;
+      MessageFind find;
+      GstMessage *smessage = NULL;
+
+      GST_DEBUG_OBJECT (bin, "ASYNC_DONE message %p, %s", message,
+          GST_OBJECT_NAME (src));
+
+      GST_OBJECT_LOCK (bin);
+      target = GST_STATE_TARGET (bin);
+      /* ignore messages if we are shutting down */
+      if (target <= GST_STATE_READY)
+        goto ignore_done_message;
+
+      bin_replace_message (bin, message, GST_MESSAGE_ASYNC_START);
+      /* if there are no more ASYNC_START messages, everybody posted
+       * a ASYNC_DONE and we can post one on the bus. */
+
+      /* we don't care who still has a pending ASYNC_START */
+      find.src = NULL;
+      find.types = GST_MESSAGE_ASYNC_START;
+
+      if (!g_list_find_custom (bin->messages, &find,
+              (GCompareFunc) message_check)) {
+        /* nothing found, remove all old ASYNC_DONE messages */
+        bin_remove_messages (bin, NULL, GST_MESSAGE_ASYNC_DONE);
+        done = TRUE;
+        toplevel = bin->priv->asynchandling
+            || (GST_OBJECT_PARENT (bin) == NULL);
+        if (!toplevel)
+          bin_handle_async_done (bin, &smessage);
+      }
+      GST_OBJECT_UNLOCK (bin);
+
+      if (smessage) {
+        GST_DEBUG_OBJECT (bin, "posting state change message");
+        gst_element_post_message (GST_ELEMENT_CAST (bin), smessage);
+      }
+      if (done) {
+        if (!toplevel) {
+          GST_DEBUG_OBJECT (bin,
+              "all async-done, posting ASYNC_DONE to parent");
+          /* post our combined ASYNC_DONE when all is ASYNC_DONE. */
+          gst_element_post_message (GST_ELEMENT_CAST (bin),
+              gst_message_new_async_done (GST_OBJECT_CAST (bin)));
+        } else {
+          /* toplevel, start continue state */
+          GST_DEBUG_OBJECT (bin, "all async-done, starting state continue");
+          bin_push_state_continue (bin);
+        }
+      }
+      break;
+
+    ignore_done_message:
+      {
+        GST_DEBUG_OBJECT (bin, "ignoring message, target %s",
+            gst_element_state_get_name (target));
+        GST_OBJECT_UNLOCK (bin);
+        gst_message_unref (message);
+        break;
+      }
+    }
     default:
       goto forward;
   }
@@ -2336,7 +2891,7 @@ typedef struct
 typedef void (*QueryInitFunction) (GstBin * bin, QueryFold * fold);
 typedef void (*QueryDoneFunction) (GstBin * bin, QueryFold * fold);
 
-/* for duration/position we collect all durations/positions and take 
+/* for duration/position we collect all durations/positions and take
  * the MAX of all valid results */
 static void
 bin_query_min_max_init (GstBin * bin, QueryFold * fold)
@@ -2554,8 +3109,8 @@ gst_bin_query (GstElement * element, GstQuery * query)
   g_value_set_boolean (&ret, FALSE);
 
   iter = gst_bin_iterate_sinks (bin);
-  GST_DEBUG_OBJECT (bin, "Sending query %p (type %d) to sink children",
-      query, GST_QUERY_TYPE (query));
+  GST_DEBUG_OBJECT (bin, "Sending query %p (type %s) to sink children",
+      query, GST_QUERY_TYPE_NAME (query));
 
   if (fold_init)
     fold_init (bin, &fold_data);
