@@ -54,6 +54,7 @@ typedef enum {
 
 typedef struct _GstBin GstBin;
 typedef struct _GstBinClass GstBinClass;
+typedef struct _GstBinPrivate GstBinPrivate;
 
 /**
  * GST_BIN_NUMCHILDREN:
@@ -83,7 +84,7 @@ typedef struct _GstBinClass GstBinClass;
  * @children: the list of children in this bin
  * @children_cookie: updated whenever @children changes
  * @child_bus: internal bus for handling child messages
- * @messages: queued messages
+ * @messages: queued and cached messages
  * @polling: the bin is currently calculating its state
  * @state_dirty: the bin needs to recalculate its state
  * @clock_dirty: the bin needs to select a new clock
@@ -113,8 +114,10 @@ struct _GstBin {
   GstClock	*provided_clock;
   GstElement    *clock_provider;
 
+  GstBinPrivate *priv;
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  gpointer _gst_reserved[GST_PADDING - 1];
 };
 
 /**
@@ -161,7 +164,7 @@ gboolean	gst_bin_remove			(GstBin *bin, GstElement *element);
 /* retrieve a single child */
 GstElement*	gst_bin_get_by_name		 (GstBin *bin, const gchar *name);
 GstElement*	gst_bin_get_by_name_recurse_up	 (GstBin *bin, const gchar *name);
-GstElement*	gst_bin_get_by_interface	 (GstBin *bin, GType interface);
+GstElement*	gst_bin_get_by_interface	 (GstBin *bin, GType iface);
 
 /* retrieve multiple children */
 GstIterator*    gst_bin_iterate_elements	 (GstBin *bin);
@@ -170,7 +173,7 @@ GstIterator*    gst_bin_iterate_recurse		 (GstBin *bin);
 
 GstIterator*	gst_bin_iterate_sinks		 (GstBin *bin);
 GstIterator*	gst_bin_iterate_sources		 (GstBin *bin);
-GstIterator*	gst_bin_iterate_all_by_interface (GstBin *bin, GType interface);
+GstIterator*	gst_bin_iterate_all_by_interface (GstBin *bin, GType iface);
 
 G_END_DECLS
 
