@@ -271,6 +271,10 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
       g_type_name (type));
 
   klass = GST_ELEMENT_CLASS (g_type_class_ref (type));
+  if ((klass->details.longname == NULL) ||
+      (klass->details.klass == NULL) || (klass->details.author == NULL))
+    goto detailserror;
+
   factory->type = type;
   __gst_element_details_copy (&factory->details, &klass->details);
   for (item = klass->padtemplates; item; item = item->next) {
@@ -294,13 +298,13 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
         g_type_interface_peek (klass, GST_TYPE_URI_HANDLER);
 
     if (!iface || !iface->get_type || !iface->get_protocols)
-      goto error;
+      goto urierror;
     factory->uri_type = iface->get_type ();
     if (!GST_URI_TYPE_IS_VALID (factory->uri_type))
-      goto error;
+      goto urierror;
     factory->uri_protocols = g_strdupv (iface->get_protocols ());
     if (!factory->uri_protocols)
-      goto error;
+      goto urierror;
   }
 
   interfaces = g_type_interfaces (type, &n_interfaces);
@@ -309,7 +313,11 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   }
   g_free (interfaces);
 
-  GST_PLUGIN_FEATURE (factory)->plugin_name = g_strdup (plugin->desc.name);
+  if (plugin && plugin->desc.name) {
+    GST_PLUGIN_FEATURE (factory)->plugin_name = g_strdup (plugin->desc.name);
+  } else {
+    GST_PLUGIN_FEATURE (factory)->plugin_name = g_strdup ("NULL");
+  }
   gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
   GST_PLUGIN_FEATURE (factory)->loaded = TRUE;
 
@@ -319,12 +327,20 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   return TRUE;
 
   /* ERRORS */
-error:
+urierror:
   {
     GST_WARNING_OBJECT (factory, "error with uri handler!");
     gst_element_factory_cleanup (factory);
     return FALSE;
   }
+
+detailserror:
+  {
+    GST_WARNING_OBJECT (factory,
+        "The GstElementDetails don't seem to have been set properly");
+    gst_element_factory_cleanup (factory);
+    return FALSE;
+  }
 }
 
 /**
@@ -347,15 +363,13 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
 
   g_return_val_if_fail (factory != NULL, NULL);
 
-  gst_object_ref (factory);
-
   newfactory =
       GST_ELEMENT_FACTORY (gst_plugin_feature_load (GST_PLUGIN_FEATURE
           (factory)));
+
   if (newfactory == NULL)
     goto load_failed;
 
-  gst_object_unref (factory);
   factory = newfactory;
 
   if (name)
@@ -369,14 +383,18 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
 
   /* create an instance of the element, cast so we don't assert on NULL */
   element = GST_ELEMENT_CAST (g_object_new (factory->type, NULL));
-  if (element == NULL)
+  if (G_UNLIKELY (element == NULL))
     goto no_element;
 
   /* fill in the pointer to the factory in the element class. The
-   * class will not be unreffed currently. */
+   * class will not be unreffed currently. 
+   * FIXME: This isn't safe and may leak a refcount on the factory if 2 threads
+   * create the first instance of an element at the same moment */
   oclass = GST_ELEMENT_GET_CLASS (element);
-  if (oclass->elementfactory == NULL)
+  if (G_UNLIKELY (oclass->elementfactory == NULL))
     oclass->elementfactory = factory;
+  else
+    gst_object_unref (factory);
 
   if (name)
     gst_object_set_name (GST_OBJECT (element), name);
@@ -394,11 +412,13 @@ load_failed:
 no_type:
   {
     GST_WARNING_OBJECT (factory, "factory has no type");
+    gst_object_unref (factory);
     return NULL;
   }
 no_element:
   {
     GST_WARNING_OBJECT (factory, "could not create element");
+    gst_object_unref (factory);
     return NULL;
   }
 }
@@ -586,7 +606,7 @@ __gst_element_factory_add_interface (GstElementFactory * elementfactory,
  * gst_element_factory_get_static_pad_templates:
  * @factory: a #GstElementFactory
  *
- * Gets the #GList of padtemplates for this factory.
+ * Gets the #GList of #GstStaticPadTemplate for this factory.
  *
  * Returns: the padtemplates
  */
@@ -618,9 +638,10 @@ gst_element_factory_get_uri_type (GstElementFactory * factory)
  * gst_element_factory_get_uri_protocols:
  * @factory: a #GstElementFactory
  *
- * Gets a NULL-terminated array of protocols this element supports or NULL, if
+ * Gets a NULL-terminated array of protocols this element supports or NULL if
  * no protocols are supported. You may not change the contents of the returned
- * array as it is still ownt by the element factory. Use g_strdupv() if you want to.
+ * array, as it is still owned by the element factory. Use g_strdupv() to
+ * make a copy of the protocol string array if you need to.
  *
  * Returns: the supported protocols or NULL
  */
