@@ -30,21 +30,33 @@ typedef struct _GstMessageClass GstMessageClass;
 /**
  * GstMessageType:
  * @GST_MESSAGE_UNKNOWN: an undefined message
- * @GST_MESSAGE_EOS: end-of-stream reached in a pipeline
- * @GST_MESSAGE_ERROR: an error occured
+ * @GST_MESSAGE_EOS: end-of-stream reached in a pipeline. The application will
+ * only receive this message in the PLAYING state and every time it sets a
+ * pipeline to PLAYING that is in the EOS state. The application can perform a
+ * seek in the pipeline to a new position.
+ * @GST_MESSAGE_ERROR: an error occured. Whe the application receives an error
+ * message it should stop playback of the pipeline and not assume that more
+ * data will be played.
  * @GST_MESSAGE_WARNING: a warning occured.
  * @GST_MESSAGE_INFO: an info message occured
  * @GST_MESSAGE_TAG: a tag was found.
- * @GST_MESSAGE_BUFFERING: the pipeline is buffering
+ * @GST_MESSAGE_BUFFERING: the pipeline is buffering. When the application
+ * receives a buffering message in the PLAYING state for a non-live pipeline it
+ * must PAUSE the pipeline until the buffering completes, when the percentage
+ * field in the message is 100%. For live pipelines, no action must be
+ * performed and the buffering percentage can be used to infor the user about
+ * the progress.
  * @GST_MESSAGE_STATE_CHANGED: a state change happened
- * @GST_MESSAGE_STATE_DIRTY: an element changed state in a streaming thread
- * @GST_MESSAGE_STEP_DONE: a framestep finished.
+ * @GST_MESSAGE_STATE_DIRTY: an element changed state in a streaming thread.
+ * This message is deprecated.
+ * @GST_MESSAGE_STEP_DONE: a framestep finished. This message is not yet
+ * implemented.
  * @GST_MESSAGE_CLOCK_PROVIDE: an element notifies its capability of providing
  *                             a clock.
  * @GST_MESSAGE_CLOCK_LOST: The current clock as selected by the pipeline became
  *                          unusable. The pipeline will select a new clock on
  *                          the next PLAYING state change.
- * @GST_MESSAGE_NEW_CLOCK: a new clock was selected in the pipeline
+ * @GST_MESSAGE_NEW_CLOCK: a new clock was selected in the pipeline.
  * @GST_MESSAGE_STRUCTURE_CHANGE: the structure of the pipeline changed.
  * @GST_MESSAGE_STREAM_STATUS: status about a stream, emitted when it starts,
  *                             stops, errors, etc..
@@ -55,11 +67,19 @@ typedef struct _GstMessageClass GstMessageClass;
  * @GST_MESSAGE_SEGMENT_START: pipeline started playback of a segment.
  * @GST_MESSAGE_SEGMENT_DONE: pipeline completed playback of a segment.
  * @GST_MESSAGE_DURATION: The duration of a pipeline changed.
+ * @GST_MESSAGE_ASYNC_START: Posted by elements when they start an ASYNC state
+ * change. Since: 0.10.13
+ * @GST_MESSAGE_ASYNC_DONE: Posted by elements when they complete an ASYNC state
+ * change. Since: 0.10.13
+ * @GST_MESSAGE_LATENCY: Posted by elements when their latency changes. The
+ * pipeline will calculate and distribute a new latency. Since: 0.10.12
  * @GST_MESSAGE_ANY: mask for all of the above messages.
  *
  * The different message types that are available.
  */
-/* NOTE: keep in sync with quark registration in gstmessage.c */
+/* NOTE: keep in sync with quark registration in gstmessage.c 
+ * NOTE: keep GST_MESSAGE_ANY a valid gint to avoid compiler warnings.
+ */ 
 typedef enum
 {
   GST_MESSAGE_UNKNOWN           = 0,
@@ -82,7 +102,10 @@ typedef enum
   GST_MESSAGE_SEGMENT_START     = (1 << 16),
   GST_MESSAGE_SEGMENT_DONE      = (1 << 17),
   GST_MESSAGE_DURATION          = (1 << 18),
-  GST_MESSAGE_ANY               = 0xffffffff
+  GST_MESSAGE_LATENCY           = (1 << 19),
+  GST_MESSAGE_ASYNC_START       = (1 << 20),
+  GST_MESSAGE_ASYNC_DONE        = (1 << 21),
+  GST_MESSAGE_ANY               = ~0
 } GstMessageType;
 
 #include <gst/gstminiobject.h>
@@ -156,7 +179,7 @@ typedef enum
  * @src: the src of the message
  * @structure: the #GstStructure containing the message info.
  *
- * A #GstMessage. 
+ * A #GstMessage.
  */
 struct _GstMessage
 {
@@ -184,9 +207,9 @@ struct _GstMessageClass {
   gpointer _gst_reserved[GST_PADDING];
 };
 
-void 		_gst_message_initialize 	(void);
+void		_gst_message_initialize		(void);
 
-GType 		gst_message_get_type 		(void);
+GType		gst_message_get_type		(void);
 
 const gchar*	gst_message_type_get_name	(GstMessageType type);
 GQuark		gst_message_type_to_quark	(GstMessageType type);
@@ -196,9 +219,14 @@ GQuark		gst_message_type_to_quark	(GstMessageType type);
  * gst_message_ref:
  * @msg: the message to ref
  *
- * Convenience macro to increase the reference count of the message. Returns the
- * reffed message.
+ * Convenience macro to increase the reference count of the message.
+ *
+ * Returns: @msg (for convenience when doing assignments)
  */
+#ifdef _FOOL_GTK_DOC_
+G_INLINE_FUNC GstMessage * gst_message_ref (GstMessage * msg);
+#endif
+
 static inline GstMessage *
 gst_message_ref (GstMessage * msg)
 {
@@ -211,8 +239,8 @@ gst_message_ref (GstMessage * msg)
  * gst_message_unref:
  * @msg: the message to unref
  *
- * Convenience macro to decrease the reference count of the message, possibly freeing
- * the it.
+ * Convenience macro to decrease the reference count of the message, possibly
+ * freeing it.
  */
 #define         gst_message_unref(msg)		gst_mini_object_unref (GST_MINI_OBJECT (msg))
 /* copy message */
@@ -236,36 +264,44 @@ gst_message_ref (GstMessage * msg)
  */
 #define         gst_message_make_writable(msg)	GST_MESSAGE (gst_mini_object_make_writable (GST_MINI_OBJECT (msg)))
 
-GstMessage *	gst_message_new_eos 		(GstObject * src);
-GstMessage *	gst_message_new_error 		(GstObject * src, GError * error, gchar * debug);
-GstMessage *	gst_message_new_warning 	(GstObject * src, GError * error, gchar * debug);
-GstMessage *	gst_message_new_tag 		(GstObject * src, GstTagList * tag_list);
-GstMessage *	gst_message_new_state_changed 	(GstObject * src, GstState oldstate,
+GstMessage *	gst_message_new_eos		(GstObject * src);
+GstMessage *	gst_message_new_error		(GstObject * src, GError * error, gchar * debug);
+GstMessage *	gst_message_new_warning		(GstObject * src, GError * error, gchar * debug);
+GstMessage *	gst_message_new_info		(GstObject * src, GError * error, gchar * debug);
+GstMessage *	gst_message_new_tag		(GstObject * src, GstTagList * tag_list);
+GstMessage *	gst_message_new_buffering	(GstObject * src, gint percent);
+GstMessage *	gst_message_new_state_changed	(GstObject * src, GstState oldstate,
                                                  GstState newstate, GstState pending);
-GstMessage *	gst_message_new_state_dirty 	(GstObject * src);
+GstMessage *	gst_message_new_state_dirty	(GstObject * src);
 GstMessage *	gst_message_new_clock_provide	(GstObject * src, GstClock *clock, gboolean ready);
 GstMessage *	gst_message_new_clock_lost	(GstObject * src, GstClock *clock);
 GstMessage *	gst_message_new_new_clock	(GstObject * src, GstClock *clock);
 GstMessage *	gst_message_new_application	(GstObject * src, GstStructure * structure);
 GstMessage *	gst_message_new_element		(GstObject * src, GstStructure * structure);
-GstMessage *	gst_message_new_segment_start 	(GstObject * src, GstFormat format, gint64 position);
-GstMessage *	gst_message_new_segment_done 	(GstObject * src, GstFormat format, gint64 position);
+GstMessage *	gst_message_new_segment_start	(GstObject * src, GstFormat format, gint64 position);
+GstMessage *	gst_message_new_segment_done	(GstObject * src, GstFormat format, gint64 position);
 GstMessage *	gst_message_new_duration	(GstObject * src, GstFormat format, gint64 duration);
-GstMessage *	gst_message_new_custom 		(GstMessageType type,
+GstMessage *	gst_message_new_async_start	(GstObject * src, gboolean new_base_time);
+GstMessage *	gst_message_new_async_done	(GstObject * src);
+GstMessage *	gst_message_new_latency         (GstObject * src);
+GstMessage *	gst_message_new_custom		(GstMessageType type,
 						 GstObject    * src,
 						 GstStructure * structure);
 
 void		gst_message_parse_error		(GstMessage *message, GError **gerror, gchar **debug);
 void		gst_message_parse_warning	(GstMessage *message, GError **gerror, gchar **debug);
+void		gst_message_parse_info 		(GstMessage *message, GError **gerror, gchar **debug);
 void		gst_message_parse_tag		(GstMessage *message, GstTagList **tag_list);
+void 		gst_message_parse_buffering	(GstMessage *message, gint *percent);
 void		gst_message_parse_state_changed	(GstMessage *message, GstState *oldstate,
                                                  GstState *newstate, GstState *pending);
 void		gst_message_parse_clock_provide (GstMessage *message, GstClock **clock, gboolean *ready);
 void		gst_message_parse_clock_lost	(GstMessage *message, GstClock **clock);
 void		gst_message_parse_new_clock	(GstMessage *message, GstClock **clock);
-void 		gst_message_parse_segment_start (GstMessage *message, GstFormat *format, gint64 *position);
-void		gst_message_parse_segment_done 	(GstMessage *message, GstFormat *format, gint64 *position);
-void		gst_message_parse_duration 	(GstMessage *message, GstFormat *format, gint64 *duration);
+void		gst_message_parse_segment_start (GstMessage *message, GstFormat *format, gint64 *position);
+void		gst_message_parse_segment_done	(GstMessage *message, GstFormat *format, gint64 *position);
+void		gst_message_parse_duration	(GstMessage *message, GstFormat *format, gint64 *duration);
+void		gst_message_parse_async_start	(GstMessage *message, gboolean *new_base_time);
 
 const GstStructure *  gst_message_get_structure	(GstMessage *message);
 
