@@ -51,8 +51,8 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <glib/gstdio.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #endif
@@ -74,8 +74,8 @@
 
 #define GST_CAT_DEFAULT GST_CAT_PLUGIN_LOADING
 
-static GModule *main_module = NULL;
 static GList *_gst_plugin_static = NULL;
+static gboolean _gst_plugin_inited;
 
 /* static variables for segfault handling of plugin loading */
 static char *_gst_plugin_fault_handler_filename = NULL;
@@ -95,7 +95,7 @@ static gboolean _gst_plugin_fault_handler_is_setup = FALSE;
  * QPL: http://www.trolltech.com/licenses/qpl.html
  * MPL: http://www.opensource.org/licenses/mozilla1.1.php
  */
-static gchar *valid_licenses[] = {
+static const gchar *valid_licenses[] = {
   "LGPL",                       /* GNU Lesser General Public License */
   "GPL",                        /* GNU General Public License */
   "QPL",                        /* Trolltech Qt Public License */
@@ -107,7 +107,7 @@ static gchar *valid_licenses[] = {
 };
 
 static GstPlugin *gst_plugin_register_func (GstPlugin * plugin,
-    GModule * module, GstPluginDesc * desc);
+    GstPluginDesc * desc);
 static void
 gst_plugin_desc_copy (GstPluginDesc * dest, const GstPluginDesc * src);
 static void gst_plugin_desc_free (GstPluginDesc * desc);
@@ -115,8 +115,6 @@ static void gst_plugin_desc_free (GstPluginDesc * desc);
 
 G_DEFINE_TYPE (GstPlugin, gst_plugin, GST_TYPE_OBJECT);
 
-static GstObjectClass *parent_class = NULL;
-
 static void
 gst_plugin_init (GstPlugin * plugin)
 {
@@ -126,7 +124,7 @@ gst_plugin_init (GstPlugin * plugin)
 static void
 gst_plugin_finalize (GObject * object)
 {
-  GstPlugin *plugin = GST_PLUGIN (object);
+  GstPlugin *plugin = GST_PLUGIN_CAST (object);
   GstRegistry *registry = gst_registry_get_default ();
   GList *g;
 
@@ -140,14 +138,12 @@ gst_plugin_finalize (GObject * object)
   g_free (plugin->basename);
   gst_plugin_desc_free (&plugin->desc);
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gst_plugin_parent_class)->finalize (object);
 }
 
 static void
 gst_plugin_class_init (GstPluginClass * klass)
 {
-  parent_class = g_type_class_ref (GST_TYPE_OBJECT);
-
   G_OBJECT_CLASS (klass)->finalize = GST_DEBUG_FUNCPTR (gst_plugin_finalize);
 }
 
@@ -170,7 +166,7 @@ gst_plugin_error_quark (void)
 void
 _gst_plugin_register_static (GstPluginDesc * desc)
 {
-  if (main_module == NULL) {
+  if (!_gst_plugin_inited) {
     if (GST_CAT_DEFAULT)
       GST_LOG ("queueing static plugin \"%s\" for loading later on",
           desc->name);
@@ -181,7 +177,7 @@ _gst_plugin_register_static (GstPluginDesc * desc)
     if (GST_CAT_DEFAULT)
       GST_LOG ("attempting to load static plugin \"%s\" now...", desc->name);
     plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
-    if (gst_plugin_register_func (plugin, main_module, desc)) {
+    if (gst_plugin_register_func (plugin, desc)) {
       if (GST_CAT_DEFAULT)
         GST_INFO ("loaded static plugin \"%s\"", desc->name);
       gst_default_registry_add_plugin (plugin);
@@ -192,7 +188,7 @@ _gst_plugin_register_static (GstPluginDesc * desc)
 void
 _gst_plugin_initialize (void)
 {
-  main_module = g_module_open (NULL, G_MODULE_BIND_LAZY);
+  _gst_plugin_inited = TRUE;
 
   /* now register all static plugins */
   g_list_foreach (_gst_plugin_static, (GFunc) _gst_plugin_register_static,
@@ -206,7 +202,7 @@ _gst_plugin_initialize (void)
 static gboolean
 gst_plugin_check_license (const gchar * license)
 {
-  gchar **check_license = valid_licenses;
+  const gchar **check_license = valid_licenses;
 
   g_assert (check_license);
 
@@ -230,8 +226,7 @@ gst_plugin_check_version (gint major, gint minor)
 }
 
 static GstPlugin *
-gst_plugin_register_func (GstPlugin * plugin, GModule * module,
-    GstPluginDesc * desc)
+gst_plugin_register_func (GstPlugin * plugin, GstPluginDesc * desc)
 {
   if (!gst_plugin_check_version (desc->major_version, desc->minor_version)) {
     if (GST_CAT_DEFAULT)
@@ -273,7 +268,9 @@ gst_plugin_register_func (GstPlugin * plugin, GModule * module,
   return plugin;
 }
 
-#ifndef HAVE_WIN32
+#ifdef HAVE_SIGACTION
+static struct sigaction oldaction;
+
 /*
  * _gst_plugin_fault_handler_restore:
  * segfault handler restorer
@@ -281,12 +278,12 @@ gst_plugin_register_func (GstPlugin * plugin, GModule * module,
 static void
 _gst_plugin_fault_handler_restore (void)
 {
-  struct sigaction action;
+  if (!_gst_plugin_fault_handler_is_setup)
+    return;
 
-  memset (&action, 0, sizeof (action));
-  action.sa_handler = SIG_DFL;
+  _gst_plugin_fault_handler_is_setup = FALSE;
 
-  sigaction (SIGSEGV, &action, NULL);
+  sigaction (SIGSEGV, &oldaction, NULL);
 }
 
 /*
@@ -331,10 +328,12 @@ _gst_plugin_fault_handler_setup (void)
   if (_gst_plugin_fault_handler_is_setup)
     return;
 
+  _gst_plugin_fault_handler_is_setup = TRUE;
+
   memset (&action, 0, sizeof (action));
   action.sa_handler = _gst_plugin_fault_handler_sighandler;
 
-  sigaction (SIGSEGV, &action, NULL);
+  sigaction (SIGSEGV, &action, &oldaction);
 }
 #else
 static void
@@ -399,12 +398,12 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
     goto return_error;
   }
 
-  if (stat (filename, &file_status)) {
+  if (g_stat (filename, &file_status)) {
     GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "problem accessing file");
     g_set_error (error,
         GST_PLUGIN_ERROR,
-        GST_PLUGIN_ERROR_MODULE, "Problem accessing file %s: %s\n", filename,
-        strerror (errno));
+        GST_PLUGIN_ERROR_MODULE, "Problem accessing file %s: %s", filename,
+        g_strerror (errno));
     goto return_error;
   }
 
@@ -413,7 +412,12 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
     GST_CAT_WARNING (GST_CAT_PLUGIN_LOADING, "module_open failed: %s",
         g_module_error ());
     g_set_error (error,
-        GST_PLUGIN_ERROR, GST_PLUGIN_ERROR_MODULE, "Opening module failed");
+        GST_PLUGIN_ERROR, GST_PLUGIN_ERROR_MODULE, "Opening module failed: %s",
+        g_module_error ());
+    /* If we failed to open the shared object, then it's probably because a
+     * plugin is linked against the wrong libraries. Print out an easy-to-see
+     * message in this case. */
+    g_warning ("Failed to load plugin '%s': %s", filename, g_module_error ());
     goto return_error;
   }
 
@@ -432,6 +436,7 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
         GST_PLUGIN_ERROR,
         GST_PLUGIN_ERROR_MODULE,
         "File \"%s\" is not a GStreamer plugin", filename);
+    g_module_close (module);
     goto return_error;
   }
   plugin->orig_desc = (GstPluginDesc *) ptr;
@@ -446,7 +451,7 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
   GST_LOG ("Plugin %p for file \"%s\" prepared, registering...",
       plugin, filename);
 
-  if (!gst_plugin_register_func (plugin, module, plugin->orig_desc)) {
+  if (!gst_plugin_register_func (plugin, plugin->orig_desc)) {
     /* remove signal handler */
     _gst_plugin_fault_handler_restore ();
     GST_DEBUG ("gst_plugin_register_func failed for plugin \"%s\"", filename);
@@ -874,7 +879,7 @@ gst_plugin_load_by_name (const gchar * name)
     gst_object_unref (plugin);
 
     if (!newplugin) {
-      GST_WARNING ("load_plugin error: %s\n", error->message);
+      GST_WARNING ("load_plugin error: %s", error->message);
       g_error_free (error);
       return NULL;
     }
@@ -920,7 +925,7 @@ gst_plugin_load (GstPlugin * plugin)
 
 load_error:
   {
-    GST_WARNING ("load_plugin error: %s\n", error->message);
+    GST_WARNING ("load_plugin error: %s", error->message);
     g_error_free (error);
     return NULL;
   }
@@ -938,7 +943,7 @@ gst_plugin_list_free (GList * list)
   GList *g;
 
   for (g = list; g; g = g->next) {
-    gst_object_unref (GST_PLUGIN (g->data));
+    gst_object_unref (GST_PLUGIN_CAST (g->data));
   }
   g_list_free (list);
 }
