@@ -58,7 +58,7 @@ gst_uri_handler_get_type (void)
 {
   static GType urihandler_type = 0;
 
-  if (!urihandler_type) {
+  if (G_UNLIKELY (urihandler_type == 0)) {
     static const GTypeInfo urihandler_info = {
       sizeof (GstURIHandlerInterface),
       gst_uri_handler_base_init,
@@ -369,7 +369,7 @@ gst_uri_get_protocol (const gchar * uri)
 
   colon = strstr (uri, "://");
 
-  return g_strndup (uri, colon - uri);
+  return g_strdown (g_strndup (uri, colon - uri));
 }
 
 /**
@@ -377,7 +377,9 @@ gst_uri_get_protocol (const gchar * uri)
  * @uri: an URI string
  * @protocol: a protocol string (e.g. "http")
  *
- * Returns TRUE if the protocol of a given valid URI matches @protocol.
+ * Checks if the protocol of a given valid URI matches @protocol.
+ *
+ * Returns: %TRUE if the protocol matches.
  *
  * Since: 0.10.4
  */
@@ -456,7 +458,7 @@ gst_uri_construct (const gchar * protocol, const gchar * location)
 typedef struct
 {
   GstURIType type;
-  gchar *protocol;
+  const gchar *protocol;
 }
 SearchEntry;
 
@@ -475,10 +477,15 @@ search_by_entry (GstPluginFeature * feature, gpointer search_entry)
     return FALSE;
 
   protocols = gst_element_factory_get_uri_protocols (factory);
-  /* must be set when uri type is valid */
-  g_assert (protocols);
+
+  if (protocols == NULL) {
+    g_warning ("Factory '%s' implements GstUriHandler interface but returned "
+        "no supported protocols!", gst_plugin_feature_get_name (feature));
+    return FALSE;
+  }
+
   while (*protocols != NULL) {
-    if (strcmp (*protocols, entry->protocol) == 0)
+    if (g_ascii_strcasecmp (*protocols, entry->protocol) == 0)
       return TRUE;
     protocols++;
   }
@@ -495,9 +502,55 @@ sort_by_rank (gconstpointer a, gconstpointer b)
       gst_plugin_feature_get_rank (first);
 }
 
+static GList *
+get_element_factories_from_uri_protocol (const GstURIType type,
+    const gchar * protocol)
+{
+  GList *possibilities;
+  SearchEntry entry;
+
+  g_return_val_if_fail (protocol, NULL);
+
+  entry.type = type;
+  entry.protocol = protocol;
+  possibilities = gst_registry_feature_filter (gst_registry_get_default (),
+      search_by_entry, FALSE, &entry);
+
+  return possibilities;
+}
+
+/**
+ * gst_uri_protocol_is_supported:
+ * @type: Whether to check for a source or a sink
+ * @protocol: Protocol that should be checked for (e.g. "http" or "smb")
+ *
+ * Checks if an element exists that supports the given URI protocol. Note
+ * that a positive return value does not imply that a subsequent call to
+ * gst_element_make_from_uri() is guaranteed to work.
+ *
+ * Returns: TRUE
+ *
+ * Since: 0.10.13
+*/
+gboolean
+gst_uri_protocol_is_supported (const GstURIType type, const gchar * protocol)
+{
+  GList *possibilities;
+
+  g_return_val_if_fail (protocol, FALSE);
+
+  possibilities = get_element_factories_from_uri_protocol (type, protocol);
+
+  if (possibilities) {
+    g_list_free (possibilities);
+    return TRUE;
+  } else
+    return FALSE;
+}
+
 /**
  * gst_element_make_from_uri:
- * @type: Wether to create a source or a sink
+ * @type: Whether to create a source or a sink
  * @uri: URI to create an element for
  * @elementname: Name of created element, can be NULL.
  *
@@ -510,17 +563,15 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
     const gchar * elementname)
 {
   GList *possibilities, *walk;
-  SearchEntry entry;
+  gchar *protocol;
   GstElement *ret = NULL;
 
   g_return_val_if_fail (GST_URI_TYPE_IS_VALID (type), NULL);
   g_return_val_if_fail (gst_uri_is_valid (uri), NULL);
 
-  entry.type = type;
-  entry.protocol = gst_uri_get_protocol (uri);
-  possibilities = gst_registry_feature_filter (gst_registry_get_default (),
-      search_by_entry, FALSE, &entry);
-  g_free (entry.protocol);
+  protocol = gst_uri_get_protocol (uri);
+  possibilities = get_element_factories_from_uri_protocol (type, protocol);
+  g_free (protocol);
 
   if (!possibilities) {
     GST_DEBUG ("No %s for URI '%s'", type == GST_URI_SINK ? "sink" : "source",
@@ -542,7 +593,7 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
     }
     walk = walk->next;
   }
-  g_list_free (possibilities);
+  gst_plugin_feature_list_free (possibilities);
 
   GST_LOG_OBJECT (ret, "created %s for URL '%s'",
       type == GST_URI_SINK ? "sink" : "source", uri);
@@ -609,8 +660,9 @@ gst_uri_handler_get_protocols (GstURIHandler * handler)
  *
  * Gets the currently handled URI.
  *
- * Returns: the URI currently handler by the @handler.
- * Returns NULL if there are no URI currently handled.
+ * Returns: the URI currently handled by the @handler.
+ * Returns NULL if there are no URI currently handled. The returned
+ * string must not be modified or freed.
  */
 G_CONST_RETURN gchar *
 gst_uri_handler_get_uri (GstURIHandler * handler)
