@@ -2,7 +2,7 @@
  *
  * Common code for GStreamer unittests
  *
- * Copyright (C) <2004> Thomas Vander Stichele <thomas at apestaart dot org>
+ * Copyright (C) 2004,2006 Thomas Vander Stichele <thomas at apestaart dot org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -108,7 +108,10 @@ gst_check_message_error (GstMessage * message, GstMessageType type,
   GError *error;
   gchar *debug;
 
-  fail_unless_equals_int (GST_MESSAGE_TYPE (message), type);
+  fail_unless (GST_MESSAGE_TYPE (message) == type,
+      "message is of type %s instead of expected type %s",
+      gst_message_type_get_name (GST_MESSAGE_TYPE (message)),
+      gst_message_type_get_name (type));
   gst_message_parse_error (message, &error, &debug);
   fail_unless_equals_int (error->domain, domain);
   fail_unless_equals_int (error->code, code);
@@ -247,6 +250,7 @@ gst_check_teardown_sink_pad (GstElement * element)
   /* clean up floating sink pad */
   srcpad = gst_element_get_pad (element, "src");
   sinkpad = gst_pad_get_peer (srcpad);
+
   gst_pad_unlink (srcpad, sinkpad);
 
   /* pad refs held by both creator and this function (through _get_pad) */
@@ -276,19 +280,50 @@ gst_check_abi_list (GstCheckABIStruct list[], gboolean have_abi_sizes)
     }
     fail_unless (ok, "failed ABI check");
   } else {
-    if (g_getenv ("GST_ABI")) {
+    const gchar *fn;
+
+    if ((fn = g_getenv ("GST_ABI"))) {
+      GError *err = NULL;
+      GString *s;
       gint i;
 
-      g_print ("\nGstCheckABIStruct list[] = {\n");
+      s = g_string_new ("\nGstCheckABIStruct list[] = {\n");
       for (i = 0; list[i].name; i++) {
-        g_print ("  {\"%s\", sizeof (%s), %d},\n", list[i].name, list[i].name,
-            list[i].size);
+        g_string_append_printf (s, "  {\"%s\", sizeof (%s), %d},\n",
+            list[i].name, list[i].name, list[i].size);
       }
-      g_print ("  {NULL, 0, 0}\n");
-      g_print ("};\n");
+      g_string_append (s, "  {NULL, 0, 0}\n");
+      g_string_append (s, "};\n");
+      if (!g_file_set_contents (fn, s->str, s->len, &err)) {
+        g_print ("%s", s->str);
+        g_printerr ("\nFailed to write ABI information: %s\n", err->message);
+      } else {
+        g_print ("\nWrote ABI information to '%s'.\n", fn);
+      }
+      g_string_free (s, TRUE);
     } else {
       g_print ("No structure size list was generated for this architecture.\n");
       g_print ("Run with GST_ABI environment variable set to output header.\n");
     }
   }
 }
+
+gint
+gst_check_run_suite (Suite * suite, const gchar * name, const gchar * fname)
+{
+  gint nf;
+
+  SRunner *sr = srunner_create (suite);
+
+  if (g_getenv ("GST_CHECK_XML")) {
+    /* how lucky we are to have __FILE__ end in .c */
+    gchar *xmlfilename = g_strdup_printf ("%sheck.xml", fname);
+
+    srunner_set_xml (sr, xmlfilename);
+  }
+
+  srunner_run_all (sr, CK_NORMAL);
+  nf = srunner_ntests_failed (sr);
+  srunner_free (sr);
+  return nf;
+}
