@@ -78,13 +78,25 @@ GST_DEBUG_CATEGORY_EXTERN (GST_CAT_DEFAULT);
 static GObjectClass *parent_class = NULL;
 GQuark __gst_controller_key;
 
+/* property ids */
+enum
+{
+  PROP_CONTROL_RATE = 1
+};
+
+struct _GstControllerPrivate
+{
+  GstClockTime control_rate;
+  GstClockTime last_sync;
+};
 
 /* imports from gst-interpolation.c */
 
 extern GList
-    * gst_controlled_property_find_timed_value_node (GstControlledProperty *
+    * gst_controlled_property_find_control_point_node (GstControlledProperty *
     prop, GstClockTime timestamp);
 extern GstInterpolateMethod *interpolation_methods[];
+extern guint num_interpolation_methods;
 
 /* callbacks */
 
@@ -95,7 +107,7 @@ on_object_controlled_property_changed (const GObject * object, GParamSpec * arg,
   GstControlledProperty *prop = GST_CONTROLLED_PROPERTY (user_data);
   GstController *ctrl;
 
-  GST_INFO ("notify for '%s'", prop->name);
+  GST_LOG ("notify for '%s'", prop->name);
 
   ctrl = g_object_get_qdata (G_OBJECT (object), __gst_controller_key);
   g_return_if_fail (ctrl);
@@ -116,18 +128,18 @@ on_object_controlled_property_changed (const GObject * object, GParamSpec * arg,
 /* helper */
 
 /*
- * gst_timed_value_compare:
- * @p1: a pointer to a #GstTimedValue
- * @p2: a pointer to a #GstTimedValue
+ * gst_control_point_compare:
+ * @p1: a pointer to a #GstControlPoint
+ * @p2: a pointer to a #GstControlPoint
  *
- * Compare function for g_list operations that operates on two #GstTimedValue
+ * Compare function for g_list operations that operates on two #GstControlPoint
  * parameters.
  */
 static gint
-gst_timed_value_compare (gconstpointer p1, gconstpointer p2)
+gst_control_point_compare (gconstpointer p1, gconstpointer p2)
 {
-  GstClockTime ct1 = ((GstTimedValue *) p1)->timestamp;
-  GstClockTime ct2 = ((GstTimedValue *) p2)->timestamp;
+  GstClockTime ct1 = ((GstControlPoint *) p1)->timestamp;
+  GstClockTime ct2 = ((GstControlPoint *) p2)->timestamp;
 
   return ((ct1 < ct2) ? -1 : ((ct1 == ct2) ? 0 : 1));
 /* this does not produce an gint :(
@@ -136,17 +148,17 @@ gst_timed_value_compare (gconstpointer p1, gconstpointer p2)
 }
 
 /*
- * gst_timed_value_find:
- * @p1: a pointer to a #GstTimedValue
+ * gst_control_point_find:
+ * @p1: a pointer to a #GstControlPoint
  * @p2: a pointer to a #GstClockTime
  *
- * Compare function for g_list operations that operates on a #GstTimedValue and
+ * Compare function for g_list operations that operates on a #GstControlPoint and
  * a #GstClockTime.
  */
 static gint
-gst_timed_value_find (gconstpointer p1, gconstpointer p2)
+gst_control_point_find (gconstpointer p1, gconstpointer p2)
 {
-  GstClockTime ct1 = ((GstTimedValue *) p1)->timestamp;
+  GstClockTime ct1 = ((GstControlPoint *) p1)->timestamp;
   GstClockTime ct2 = *(GstClockTime *) p2;
 
   return ((ct1 < ct2) ? -1 : ((ct1 == ct2) ? 0 : 1));
@@ -171,6 +183,11 @@ gst_controlled_property_set_interpolation_mode (GstControlledProperty * self,
 {
   gboolean res = TRUE;
 
+  if (mode >= num_interpolation_methods || interpolation_methods[mode] == NULL) {
+    GST_WARNING ("interpolation mode %d invalid or not implemented yet", mode);
+    return FALSE;
+  }
+
   self->interpolation = mode;
   if (mode != GST_INTERPOLATE_USER) {
     switch (self->base) {
@@ -224,7 +241,7 @@ gst_controlled_property_set_interpolation_mode (GstControlledProperty * self,
         self->get_value_array = NULL;
     }
     if (!self->get) {           /* || !self->get_value_array) */
-      GST_WARNING ("incomplete implementation for type %d/%d:'%s'/'%s'",
+      GST_WARNING ("incomplete implementation for type %lu/%lu:'%s'/'%s'",
           self->type, self->base,
           g_type_name (self->type), g_type_name (self->base));
       res = FALSE;
@@ -328,6 +345,7 @@ gst_controlled_property_new (GObject * object, const gchar * name)
 
           g_value_set_float (&prop->default_value, tpspec->default_value);
         }
+          break;
         case G_TYPE_DOUBLE:{
           GParamSpecDouble *tpspec = G_PARAM_SPEC_DOUBLE (pspec);
 
@@ -356,7 +374,7 @@ gst_controlled_property_new (GObject * object, const gchar * name)
           GST_WARNING ("incomplete implementation for paramspec type '%s'",
               G_PARAM_SPEC_TYPE_NAME (pspec));
       }
-      /* TODO what about adding a timedval with timestamp=0 and value=default
+      /* TODO what about adding a control point with timestamp=0 and value=default
        * a bit easier for interpolators, example:
        * first timestamp is at 5
        * requested value if for timestamp=3
@@ -381,6 +399,23 @@ Error:
 }
 
 /*
+ * gst_control_point_free:
+ * @prop: the object to free
+ *
+ * Private method which frees all data allocated by a #GstControlPoint
+ * instance.
+ */
+static void
+gst_control_point_free (GstControlPoint * cp)
+{
+  g_return_if_fail (cp);
+
+  g_value_unset (&cp->value);
+  g_free (cp);
+}
+
+/*
+
  * gst_controlled_property_free:
  * @prop: the object to free
  *
@@ -390,12 +425,15 @@ Error:
 static void
 gst_controlled_property_free (GstControlledProperty * prop)
 {
-  GList *node;
-
-  for (node = prop->values; node; node = g_list_next (node)) {
-    g_free (node->data);
-  }
+  g_list_foreach (prop->values, (GFunc) gst_control_point_free, NULL);
   g_list_free (prop->values);
+
+  g_value_unset (&prop->default_value);
+  g_value_unset (&prop->result_value);
+  g_value_unset (&prop->last_value.value);
+  if (G_IS_VALUE (&prop->live_value.value))
+    g_value_unset (&prop->live_value.value);
+
   g_free (prop);
 }
 
@@ -437,13 +475,13 @@ gst_controller_find_controlled_property (GstController * self,
  * Creates a new GstController for the given object's properties
  *
  * Returns: the new controller.
- * Since: 0.9
  */
 GstController *
 gst_controller_new_valist (GObject * object, va_list var_args)
 {
   GstController *self;
   GstControlledProperty *prop;
+  gboolean ref_existing = TRUE;
   gchar *name;
 
   g_return_val_if_fail (G_IS_OBJECT (object), NULL);
@@ -471,16 +509,26 @@ gst_controller_new_valist (GObject * object, va_list var_args)
         /* if we don't have a controller object yet, now is the time to create one */
         if (!self) {
           self = g_object_new (GST_TYPE_CONTROLLER, NULL);
-          self->object = object;
+          self->object = g_object_ref (object);
           /* store the controller */
           g_object_set_qdata (object, __gst_controller_key, self);
+          ref_existing = FALSE;
         } else {
-          GST_INFO ("returning existing controller");
+          /* only want one single _ref(), even for multiple properties */
+          if (ref_existing) {
+            g_object_ref (self);
+            ref_existing = FALSE;
+            GST_INFO ("returning existing controller");
+          }
         }
         self->properties = g_list_prepend (self->properties, prop);
       }
     } else {
       GST_WARNING ("trying to control property again");
+      if (ref_existing) {
+        g_object_ref (self);
+        ref_existing = FALSE;
+      }
     }
   }
   va_end (var_args);
@@ -498,13 +546,13 @@ gst_controller_new_valist (GObject * object, va_list var_args)
  * Creates a new GstController for the given object's properties
  *
  * Returns: the new controller.
- * Since: 0.9
  */
 GstController *
 gst_controller_new_list (GObject * object, GList * list)
 {
   GstController *self;
   GstControlledProperty *prop;
+  gboolean ref_existing = TRUE;
   gchar *name;
   GList *node;
 
@@ -523,16 +571,26 @@ gst_controller_new_list (GObject * object, GList * list)
         /* if we don't have a controller object yet, now is the time to create one */
         if (!self) {
           self = g_object_new (GST_TYPE_CONTROLLER, NULL);
-          self->object = object;
+          self->object = g_object_ref (object);
           /* store the controller */
           g_object_set_qdata (object, __gst_controller_key, self);
+          ref_existing = FALSE;
         } else {
-          GST_INFO ("returning existing controller");
+          /* only want one single _ref(), even for multiple properties */
+          if (ref_existing) {
+            g_object_ref (self);
+            ref_existing = FALSE;
+            GST_INFO ("returning existing controller");
+          }
         }
         self->properties = g_list_prepend (self->properties, prop);
       }
     } else {
       GST_WARNING ("trying to control property again");
+      if (ref_existing) {
+        g_object_ref (self);
+        ref_existing = FALSE;
+      }
     }
   }
 
@@ -549,7 +607,6 @@ gst_controller_new_list (GObject * object, GList * list)
  * Creates a new GstController for the given object's properties
  *
  * Returns: the new controller.
- * Since: 0.9
  */
 GstController *
 gst_controller_new (GObject * object, ...)
@@ -574,7 +631,6 @@ gst_controller_new (GObject * object, ...)
  * Removes the given object properties from the controller
  *
  * Returns: %FALSE if one of the given property isn't handled by the controller, %TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_remove_properties_valist (GstController * self, va_list var_args)
@@ -609,7 +665,6 @@ gst_controller_remove_properties_valist (GstController * self, va_list var_args)
  * Removes the given object properties from the controller
  *
  * Returns: %FALSE if one of the given property isn't handled by the controller, %TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_remove_properties_list (GstController * self, GList * list)
@@ -647,7 +702,6 @@ gst_controller_remove_properties_list (GstController * self, GList * list)
  * Removes the given object properties from the controller
  *
  * Returns: %FALSE if one of the given property isn't handled by the controller, %TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_remove_properties (GstController * self, ...)
@@ -674,7 +728,6 @@ gst_controller_remove_properties (GstController * self, ...)
  * Set the value of given controller-handled property at a certain time.
  *
  * Returns: FALSE if the values couldn't be set (ex : properties not handled by controller), TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_set (GstController * self, gchar * property_name,
@@ -691,22 +744,24 @@ gst_controller_set (GstController * self, gchar * property_name,
   g_mutex_lock (self->lock);
   if ((prop = gst_controller_find_controlled_property (self, property_name))) {
     if (G_VALUE_TYPE (value) == prop->type) {
-      GstTimedValue *tv;
+      GstControlPoint *cp;
       GList *node;
 
-      /* check if a timed_value for the timestamp already exists */
+      /* check if a control point for the timestamp already exists */
       if ((node = g_list_find_custom (prop->values, &timestamp,
-                  gst_timed_value_find))) {
-        tv = node->data;
-        memcpy (&tv->value, value, sizeof (GValue));
+                  gst_control_point_find))) {
+        cp = node->data;
+        g_value_reset (&cp->value);
+        g_value_copy (value, &cp->value);
       } else {
-        /* create a new GstTimedValue */
-        tv = g_new (GstTimedValue, 1);
-        tv->timestamp = timestamp;
-        memcpy (&tv->value, value, sizeof (GValue));
+        /* create a new GstControlPoint */
+        cp = g_new0 (GstControlPoint, 1);
+        cp->timestamp = timestamp;
+        g_value_init (&cp->value, prop->type);
+        g_value_copy (value, &cp->value);
         /* and sort it into the prop->values list */
         prop->values =
-            g_list_insert_sorted (prop->values, tv, gst_timed_value_compare);
+            g_list_insert_sorted (prop->values, cp, gst_control_point_compare);
       }
       res = TRUE;
     } else {
@@ -727,7 +782,6 @@ gst_controller_set (GstController * self, gchar * property_name,
  * Sets multiple timed values at once.
  *
  * Returns: %FALSE if the values couldn't be set (ex : properties not handled by controller), %TRUE otherwise
- * Since: 0.9
  */
 
 gboolean
@@ -738,6 +792,7 @@ gst_controller_set_from_list (GstController * self, gchar * property_name,
   GstControlledProperty *prop;
   GSList *node;
   GstTimedValue *tv;
+  GstControlPoint *cp;
 
   g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
   g_return_val_if_fail (property_name, FALSE);
@@ -747,11 +802,19 @@ gst_controller_set_from_list (GstController * self, gchar * property_name,
     for (node = timedvalues; node; node = g_slist_next (node)) {
       tv = node->data;
       if (G_VALUE_TYPE (&tv->value) == prop->type) {
-        g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (tv->timestamp), FALSE);
-        /* TODO copy the timed value or just link in? */
-        prop->values =
-            g_list_insert_sorted (prop->values, tv, gst_timed_value_compare);
-        res = TRUE;
+        if (GST_CLOCK_TIME_IS_VALID (tv->timestamp)) {
+          cp = g_new0 (GstControlPoint, 1);
+          cp->timestamp = tv->timestamp;
+          g_value_init (&cp->value, prop->type);
+          g_value_copy (&tv->value, &cp->value);
+          prop->values =
+              g_list_insert_sorted (prop->values, cp,
+              gst_control_point_compare);
+          res = TRUE;
+        } else {
+          g_warning ("GstTimedValued with invalid timestamp passed to %s "
+              "for property '%s'", GST_FUNCTION, property_name);
+        }
       } else {
         GST_WARNING ("incompatible value type for property '%s'", prop->name);
       }
@@ -772,7 +835,6 @@ gst_controller_set_from_list (GstController * self, gchar * property_name,
  * time.
  *
  * Returns: %FALSE if the values couldn't be unset (ex : properties not handled by controller), %TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_unset (GstController * self, gchar * property_name,
@@ -789,10 +851,12 @@ gst_controller_unset (GstController * self, gchar * property_name,
   if ((prop = gst_controller_find_controlled_property (self, property_name))) {
     GList *node;
 
-    /* check if a timed_value for the timestamp exists */
+    /* check if a control point for the timestamp exists */
     if ((node = g_list_find_custom (prop->values, &timestamp,
-                gst_timed_value_find))) {
-      g_free (node->data);      /* free GstTimedValue */
+                gst_control_point_find))) {
+      if (node == prop->last_requested_value)
+        prop->last_requested_value = NULL;
+      gst_control_point_free (node->data);      /* free GstControlPoint */
       prop->values = g_list_delete_link (prop->values, node);
       res = TRUE;
     }
@@ -824,9 +888,10 @@ gst_controller_unset_all (GstController * self, gchar * property_name)
 
   g_mutex_lock (self->lock);
   if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    /* free GstTimedValue structures */
-    g_list_foreach (prop->values, (GFunc) g_free, NULL);
+    /* free GstControlPoint structures */
+    g_list_foreach (prop->values, (GFunc) gst_control_point_free, NULL);
     g_list_free (prop->values);
+    prop->last_requested_value = NULL;
     prop->values = NULL;
     res = TRUE;
   }
@@ -845,7 +910,6 @@ gst_controller_unset_all (GstController * self, gchar * property_name)
  * time.
  *
  * Returns: the GValue of the property at the given time, or %NULL if the property isn't handled by the controller
- * Since: 0.9
  */
 GValue *
 gst_controller_get (GstController * self, gchar * property_name,
@@ -877,7 +941,6 @@ gst_controller_get (GstController * self, gchar * property_name,
  * Free the list after done with it.
  *
  * Returns: a copy of the list, or %NULL if the property isn't handled by the controller
- * Since: 0.9
  */
 const GList *
 gst_controller_get_all (GstController * self, gchar * property_name)
@@ -897,6 +960,38 @@ gst_controller_get_all (GstController * self, gchar * property_name)
   return (res);
 }
 
+/**
+ * gst_controller_suggest_next_sync:
+ * @self: the controller that handles the values
+ *
+ * Returns a suggestion for timestamps where buffers should be split
+ * to get best controller results.
+ *
+ * Returns: Returns the suggested timestamp or %GST_CLOCK_TIME_NONE
+ * if no control-rate was set.
+ *
+ * Since: 0.10.13
+ */
+GstClockTime
+gst_controller_suggest_next_sync (GstController * self)
+{
+  GstClockTime ret;
+
+  g_return_val_if_fail (GST_IS_CONTROLLER (self), GST_CLOCK_TIME_NONE);
+  g_return_val_if_fail (self->priv->control_rate != GST_CLOCK_TIME_NONE,
+      GST_CLOCK_TIME_NONE);
+
+  g_mutex_lock (self->lock);
+
+  /* TODO: Implement more logic, depending on interpolation mode
+   * and control points */
+  ret = self->priv->last_sync + self->priv->control_rate;
+
+  g_mutex_unlock (self->lock);
+
+  return ret;
+}
+
 /**
  * gst_controller_sync_values:
  * @self: the controller that handles the values
@@ -907,7 +1002,6 @@ gst_controller_get_all (GstController * self, gchar * property_name)
  *
  * Returns: %TRUE if the controller values could be applied to the object
  * properties, %FALSE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_sync_values (GstController * self, GstClockTime timestamp)
@@ -915,12 +1009,12 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
   GstControlledProperty *prop;
   GList *node;
   GValue *value;
-  gboolean live;
+  gboolean live = FALSE;
 
   g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
   g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (timestamp), FALSE);
 
-  GST_INFO ("sync_values");
+  GST_LOG ("sync_values");
 
   g_mutex_lock (self->lock);
   /* go over the controlled properties of the controller */
@@ -930,16 +1024,16 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
         timestamp);
 
     live = FALSE;
-    if (G_IS_VALUE (&prop->live_value.value)) {
+    if (G_UNLIKELY (G_IS_VALUE (&prop->live_value.value))) {
       GList *lnode =
-          gst_controlled_property_find_timed_value_node (prop, timestamp);
-      if (!lnode) {
+          gst_controlled_property_find_control_point_node (prop, timestamp);
+      if (G_UNLIKELY (!lnode)) {
         GST_DEBUG ("    no control changes in the queue");
         live = TRUE;
       } else {
-        GstTimedValue *tv = lnode->data;
+        GstControlPoint *cp = lnode->data;
 
-        if (prop->live_value.timestamp < tv->timestamp) {
+        if (prop->live_value.timestamp < cp->timestamp) {
           g_value_unset (&prop->live_value.value);
           GST_DEBUG ("    live value resetted");
         } else if (prop->live_value.timestamp < timestamp) {
@@ -947,7 +1041,7 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
         }
       }
     }
-    if (!live) {
+    if (G_LIKELY (!live)) {
       /* get current value via interpolator */
       value = prop->get (prop, timestamp);
       prop->last_value.timestamp = timestamp;
@@ -955,6 +1049,9 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
       g_object_set_property (self->object, prop->name, value);
     }
   }
+  if (G_LIKELY (!live))
+    self->priv->last_sync = timestamp;
+
   g_mutex_unlock (self->lock);
   /* TODO what can here go wrong, to return FALSE ?
      BilboEd : Nothing I guess, as long as all the checks are made when creating the controller,
@@ -973,12 +1070,13 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
  * Function to be able to get an array of values for one or more given element
  * properties.
  *
- * If the GstValueArray->values array in list nodes is NULL, it will be created
- * by the function.
- * The type of the values in the array are the same as the property's type.
+ * All fields of the %GstValueArray in the list must be filled correctly.
+ * Especially the GstValueArray->values arrays must be big enough to keep
+ * the requested amount of values.
+ *
+ * The types of the values in the array are the same as the property's type.
  *
  * Returns: %TRUE if the given array(s) could be filled, %FALSE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_get_value_arrays (GstController * self,
@@ -1004,13 +1102,15 @@ gst_controller_get_value_arrays (GstController * self,
  * @timestamp: the time that should be processed
  * @value_array: array to put control-values in
  *
- * Function to be able to get an array of values for one element properties
+ * Function to be able to get an array of values for one element property.
  *
- * If the GstValueArray->values array is NULL, it will be created by the function.
- * The type of the values in the array are the same as the property's type.
+ * All fields of @value_array must be filled correctly. Especially the
+ * @value_array->values array must be big enough to keep the requested amount
+ * of values.
  *
- * Returns: %TRUE if the given array(s) could be filled, %FALSE otherwise
- * Since: 0.9
+ * The type of the values in the array is the same as the property's type.
+ *
+ * Returns: %TRUE if the given array could be filled, %FALSE otherwise
  */
 gboolean
 gst_controller_get_value_array (GstController * self, GstClockTime timestamp,
@@ -1023,23 +1123,17 @@ gst_controller_get_value_array (GstController * self, GstClockTime timestamp,
   g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (timestamp), FALSE);
   g_return_val_if_fail (value_array, FALSE);
   g_return_val_if_fail (value_array->property_name, FALSE);
-
-  /* TODO and if GstValueArray->values is not NULL, the caller is resposible that
-     is is big enough for nbsamples values, right?
-   */
+  g_return_val_if_fail (value_array->values, FALSE);
 
   g_mutex_lock (self->lock);
+
   if ((prop =
           gst_controller_find_controlled_property (self,
               value_array->property_name))) {
-    if (!value_array->values) {
-      /* TODO from where to get the base size
-         value_array->values=g_new(sizeof(???),nbsamples);
-       */
-    }
     /* get current value_array via interpolator */
     res = prop->get_value_array (prop, timestamp, value_array);
   }
+
   g_mutex_unlock (self->lock);
   return (res);
 }
@@ -1052,8 +1146,10 @@ gst_controller_get_value_array (GstController * self, GstClockTime timestamp,
  *
  * Sets the given interpolation mode on the given property.
  *
+ * <note><para>Quadratic, cubic and user interpolation is not yet available.
+ * </para></note>
+ *
  * Returns: %TRUE if the property is handled by the controller, %FALSE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_set_interpolation_mode (GstController * self,
@@ -1070,58 +1166,109 @@ gst_controller_set_interpolation_mode (GstController * self,
     /* TODO shouldn't this also get a GstInterpolateMethod *user_method
        for the case mode==GST_INTERPOLATE_USER
      */
-    gst_controlled_property_set_interpolation_mode (prop, mode);
-    res = TRUE;
+    res = gst_controlled_property_set_interpolation_mode (prop, mode);
   }
   g_mutex_unlock (self->lock);
 
   return (res);
 }
 
-/*
-void
-gst_controller_set_live_value(GstController * self, gchar *property_name,
-    GstClockTime timestamp, GValue *value)
+/* gobject handling */
+
+static void
+_gst_controller_get_property (GObject * object, guint property_id,
+    GValue * value, GParamSpec * pspec)
 {
-  GstControlledProperty *prop;
+  GstController *self = GST_CONTROLLER (object);
 
-  g_return_if_fail (GST_IS_CONTROLLER (self));
-  g_return_if_fail (property_name);
+  switch (property_id) {
+    case PROP_CONTROL_RATE:{
+      /* FIXME: don't change if element is playing, controller works for GObject
+         so this wont work
 
-  g_mutex_lock (self->lock);
-  if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    g_value_unset (&prop->live_value.value);
-    g_value_init (&prop->live_value.value, prop->type);
-    g_value_copy (value, &prop->live_value.value);
-    prop->live_value.timestamp = timestamp;
+         GstState c_state, p_state;
+         GstStateChangeReturn ret;
+
+         ret = gst_element_get_state (self->object, &c_state, &p_state, 0);
+         if ((ret == GST_STATE_CHANGE_SUCCESS &&
+         (c_state == GST_STATE_NULL || c_state == GST_STATE_READY)) ||
+         (ret == GST_STATE_CHANGE_ASYNC &&
+         (p_state == GST_STATE_NULL || p_state == GST_STATE_READY))) {
+       */
+      g_value_set_uint64 (value, self->priv->control_rate);
+      /*
+         }
+         else {
+         GST_WARNING ("Changing the control rate is only allowed if the elemnt"
+         " is in NULL or READY");
+         }
+       */
+    }
+      break;
+    default:{
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+      break;
   }
-  g_mutex_unlock (self->lock);
 }
 
-*/
+/* sets the given properties for this object */
+static void
+_gst_controller_set_property (GObject * object, guint property_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstController *self = GST_CONTROLLER (object);
 
-/* gobject handling */
+  switch (property_id) {
+    case PROP_CONTROL_RATE:{
+      self->priv->control_rate = g_value_get_uint64 (value);
+    }
+      break;
+    default:{
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+      break;
+  }
+}
+
+static void
+_gst_controller_dispose (GObject * object)
+{
+  GstController *self = GST_CONTROLLER (object);
+
+  if (self->object != NULL) {
+    g_mutex_lock (self->lock);
+    /* free list of properties */
+    if (self->properties) {
+      GList *node;
+
+      for (node = self->properties; node; node = g_list_next (node)) {
+        GstControlledProperty *prop = node->data;
+
+        g_signal_handler_disconnect (self->object, prop->notify_handler_id);
+        gst_controlled_property_free (prop);
+      }
+      g_list_free (self->properties);
+      self->properties = NULL;
+    }
+
+    /* remove controller from object's qdata list */
+    g_object_set_qdata (self->object, __gst_controller_key, NULL);
+    g_object_unref (self->object);
+    self->object = NULL;
+    g_mutex_unlock (self->lock);
+  }
+
+  if (G_OBJECT_CLASS (parent_class)->dispose)
+    (G_OBJECT_CLASS (parent_class)->dispose) (object);
+}
 
 static void
 _gst_controller_finalize (GObject * object)
 {
   GstController *self = GST_CONTROLLER (object);
-  GList *node;
-  GstControlledProperty *prop;
 
-  /* free list of properties */
-  if (self->properties) {
-    for (node = self->properties; node; node = g_list_next (node)) {
-      prop = node->data;
-      g_signal_handler_disconnect (self->object, prop->notify_handler_id);
-      gst_controlled_property_free (prop);
-    }
-    g_list_free (self->properties);
-    self->properties = NULL;
-  }
   g_mutex_free (self->lock);
-  /* remove controller from objects qdata list */
-  g_object_set_qdata (self->object, __gst_controller_key, NULL);
 
   if (G_OBJECT_CLASS (parent_class)->finalize)
     (G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -1133,7 +1280,11 @@ _gst_controller_init (GTypeInstance * instance, gpointer g_class)
   GstController *self = GST_CONTROLLER (instance);
 
   self->lock = g_mutex_new ();
-
+  self->priv =
+      G_TYPE_INSTANCE_GET_PRIVATE (self, GST_TYPE_CONTROLLER,
+      GstControllerPrivate);
+  self->priv->last_sync = GST_CLOCK_TIME_NONE;
+  self->priv->control_rate = 100 * GST_MSECOND;
 }
 
 static void
@@ -1142,17 +1293,24 @@ _gst_controller_class_init (GstControllerClass * klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
+  g_type_class_add_private (klass, sizeof (GstControllerPrivate));
 
+  gobject_class->set_property = _gst_controller_set_property;
+  gobject_class->get_property = _gst_controller_get_property;
+  gobject_class->dispose = _gst_controller_dispose;
   gobject_class->finalize = _gst_controller_finalize;
 
   __gst_controller_key = g_quark_from_string ("gst::controller");
 
   /* register properties */
+  g_object_class_install_property (gobject_class, PROP_CONTROL_RATE,
+      g_param_spec_uint64 ("control-rate",
+          "control rate",
+          "Controlled properties will be updated at least every control-rate nanoseconds",
+          1, G_MAXUINT, 100 * GST_MSECOND, G_PARAM_READWRITE));
+
   /* register signals */
   /* set defaults for overridable methods */
-  /* TODO which of theses do we need ?
-     BilboEd : none :)
-   */
 }
 
 GType
