@@ -19,15 +19,20 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:element-capsfilter
+ * @short_description: pass data though unmodified, but enforces format limmits
+ *
+ * The element does not modify data as such, but can enforce limmitations on the
+ * data format.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "../../gst/gst-i18n-lib.h"
-#include <gst/gst.h>
-#include <gst/base/gstbasetransform.h>
-
+#include "gstcapsfilter.h"
 
 static const GstElementDetails gst_capsfilter_details =
 GST_ELEMENT_DETAILS ("CapsFilter",
@@ -35,36 +40,6 @@ GST_ELEMENT_DETAILS ("CapsFilter",
     "Pass data without modification, limiting formats",
     "David Schleef <ds@schleef.org>");
 
-
-#define GST_TYPE_CAPSFILTER \
-  (gst_capsfilter_get_type())
-#define GST_CAPSFILTER(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_CAPSFILTER,GstCapsFilter))
-#define GST_CAPSFILTER_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_CAPSFILTER,GstCapsFilterClass))
-#define GST_IS_CAPSFILTER(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_CAPSFILTER))
-#define GST_IS_CAPSFILTER_CLASS(obj) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_CAPSFILTER))
-
-typedef struct _GstCapsFilter GstCapsFilter;
-typedef struct _GstCapsFilterClass GstCapsFilterClass;
-
-struct _GstCapsFilter
-{
-  GstBaseTransform trans;
-
-  GstCaps *filter_caps;
-};
-
-struct _GstCapsFilterClass
-{
-  GstBaseTransformClass trans_class;
-};
-
-GType gst_capsfilter_get_type (void);
-
-
 enum
 {
   PROP_0,
@@ -257,6 +232,9 @@ gst_capsfilter_prepare_buf (GstBaseTransform * trans, GstBuffer * input,
       g_return_val_if_fail (out_caps != NULL, GST_FLOW_ERROR);
     }
 
+    out_caps = gst_caps_make_writable (out_caps);
+    gst_caps_do_simplify (out_caps);
+
     if (gst_caps_is_fixed (out_caps) && !gst_caps_is_empty (out_caps)) {
       GST_DEBUG_OBJECT (trans, "Have fixed output caps %"
           GST_PTR_FORMAT " to apply to buffer with no caps", out_caps);
@@ -272,6 +250,8 @@ gst_capsfilter_prepare_buf (GstBaseTransform * trans, GstBuffer * input,
       if (GST_PAD_CAPS (trans->srcpad) == NULL)
         gst_pad_set_caps (trans->srcpad, out_caps);
     } else {
+      GST_DEBUG_OBJECT (trans, "Have unfixed output caps %" GST_PTR_FORMAT,
+          out_caps);
       gst_caps_unref (out_caps);
     }
   }
