@@ -246,7 +246,8 @@ print_interfaces (GType type)
 
   if (ifaces) {
     if (n_ifaces) {
-      g_print ("%sImplemented Interfaces:\n", _name);
+      g_print ("%s", _name);
+      g_print (_("Implemented Interfaces:\n"));
       iface = ifaces;
       while (*iface) {
         g_print ("%s  %s\n", _name, g_type_name (*iface));
@@ -287,15 +288,18 @@ print_element_properties_info (GstElement * element)
     if (param->flags & G_PARAM_READABLE) {
       g_object_get_property (G_OBJECT (element), param->name, &value);
       readable = TRUE;
-      g_print ((first_flag ? "readable" : ", readable"));
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("readable"));
       first_flag = FALSE;
     }
     if (param->flags & G_PARAM_WRITABLE) {
-      g_print ((first_flag ? "writable" : ", writable"));
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("writable"));
       first_flag = FALSE;
     }
     if (param->flags & GST_PARAM_CONTROLLABLE) {
-      g_print ((first_flag ? "controllable" : ", controllable"));
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("controllable"));
       first_flag = FALSE;
     }
     n_print ("\n");
@@ -452,7 +456,7 @@ print_element_properties_info (GstElement * element)
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    %d) %-16s - %s", "",
+            g_print ("\n%s%-23.23s    (%d): %-16s - %s", "",
                 _name, values[j].value, values[j].value_nick,
                 values[j].value_name);
             j++;
@@ -484,8 +488,9 @@ print_element_properties_info (GstElement * element)
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (%d): \t%s", "",
-                _name, values[j].value, values[j].value_nick);
+            g_print ("\n%s%-23.23s    (0x%08x): %-16s - %s", "",
+                _name, values[j].value, values[j].value_nick,
+                values[j].value_name);
             j++;
           }
 
@@ -504,6 +509,8 @@ print_element_properties_info (GstElement * element)
           } else {
             n_print ("%-23.23s Pointer.", "");
           }
+        } else if (param->value_type == G_TYPE_VALUE_ARRAY) {
+          n_print ("%-23.23s Array of GValues", "");
         } else {
           n_print ("%-23.23s Unknown type %ld \"%s\"", "", param->value_type,
               g_type_name (param->value_type));
@@ -599,8 +606,6 @@ print_implementation_info (GstElement * element)
   n_print ("\n");
   n_print ("Element Implementation:\n");
 
-  n_print ("  No loopfunc(), must be chain-based or not configured yet\n");
-
   n_print ("  Has change_state() function: %s\n",
       GST_DEBUG_FUNCPTR_NAME (gstelement_class->change_state));
 #ifndef GST_DISABLE_LOADSAVE
@@ -793,6 +798,9 @@ print_signal_info (GstElement * element)
         if (G_TYPE_IS_FUNDAMENTAL (query->param_types[j])) {
           g_print (",\n%s%s%s arg%d", _name, indent,
               g_type_name (query->param_types[j]), j);
+        } else if (G_TYPE_IS_ENUM (query->param_types[j])) {
+          g_print (",\n%s%s%s arg%d", _name, indent,
+              g_type_name (query->param_types[j]), j);
         } else {
           g_print (",\n%s%s%s* arg%d", _name, indent,
               g_type_name (query->param_types[j]), j);
@@ -914,8 +922,12 @@ print_element_list (gboolean print_all)
 
   gst_plugin_list_free (orig_plugins);
 
-  g_print ("\nTotal plugins: %d\nTotal features: %d\n",
-      plugincount, featurecount);
+  g_print ("\n");
+  g_print (_("Total count: "));
+  g_print (ngettext ("%d plugin", "%d plugins", plugincount), plugincount);
+  g_print (", ");
+  g_print (ngettext ("%d feature", "%d features", featurecount), featurecount);
+  g_print ("\n");
 }
 
 static void
@@ -1083,19 +1095,152 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   print_signal_info (element);
   print_children_info (element);
 
+  gst_object_unref (factory);
   if (_name[0] != '\0')
     g_free (_name);
 
   return 0;
 }
 
+
+static void
+print_plugin_automatic_install_info_codecs (GstElementFactory * factory)
+{
+  GstPadDirection direction;
+  const gchar *type_name;
+  const gchar *klass;
+  const GList *static_templates, *l;
+  GstCaps *caps = NULL;
+  guint i, num;
+
+  klass = gst_element_factory_get_klass (factory);
+  g_return_if_fail (klass != NULL);
+
+  if (strstr (klass, "Demuxer") ||
+      strstr (klass, "Decoder") ||
+      strstr (klass, "Depay") || strstr (klass, "Parser")) {
+    type_name = "decoder";
+    direction = GST_PAD_SINK;
+  } else if (strstr (klass, "Muxer") ||
+      strstr (klass, "Encoder") || strstr (klass, "Pay")) {
+    type_name = "encoder";
+    direction = GST_PAD_SRC;
+  } else {
+    return;
+  }
+
+  /* decoder/demuxer sink pads should always be static and there should only
+   * be one, the same applies to encoders/muxers and source pads */
+  static_templates = gst_element_factory_get_static_pad_templates (factory);
+  for (l = static_templates; l != NULL; l = l->next) {
+    GstStaticPadTemplate *tmpl = NULL;
+
+    tmpl = (GstStaticPadTemplate *) l->data;
+    if (tmpl->direction == direction) {
+      caps = gst_static_pad_template_get_caps (tmpl);
+      break;
+    }
+  }
+
+  if (caps == NULL) {
+    g_printerr ("Couldn't find static pad template for %s '%s'\n",
+        type_name, GST_PLUGIN_FEATURE_NAME (factory));
+    return;
+  }
+
+  caps = gst_caps_make_writable (caps);
+  num = gst_caps_get_size (caps);
+  for (i = 0; i < num; ++i) {
+    GstStructure *s;
+    gchar *s_str;
+
+    s = gst_caps_get_structure (caps, i);
+    /* remove fields that are almost always just MIN-MAX of some sort
+     * in order to make the caps look less messy */
+    gst_structure_remove_field (s, "pixel-aspect-ratio");
+    gst_structure_remove_field (s, "framerate");
+    gst_structure_remove_field (s, "channels");
+    gst_structure_remove_field (s, "width");
+    gst_structure_remove_field (s, "height");
+    gst_structure_remove_field (s, "rate");
+    gst_structure_remove_field (s, "depth");
+    gst_structure_remove_field (s, "clock-rate");
+    s_str = gst_structure_to_string (s);
+    g_print ("%s-%s\n", type_name, s_str);
+    g_free (s_str);
+  }
+  gst_caps_unref (caps);
+}
+
+static void
+print_plugin_automatic_install_info_protocols (GstElementFactory * factory)
+{
+  gchar **protocols, **p;
+
+  protocols = gst_element_factory_get_uri_protocols (factory);
+  if (protocols != NULL && *protocols != NULL) {
+    switch (gst_element_factory_get_uri_type (factory)) {
+      case GST_URI_SINK:
+        for (p = protocols; *p != NULL; ++p)
+          g_print ("urisink-%s\n", *p);
+        break;
+      case GST_URI_SRC:
+        for (p = protocols; *p != NULL; ++p)
+          g_print ("urisource-%s\n", *p);
+        break;
+      default:
+        break;
+    }
+    g_strfreev (protocols);
+  }
+}
+
+static void
+print_plugin_automatic_install_info (GstPlugin * plugin)
+{
+  const gchar *plugin_name;
+  GList *features, *l;
+
+  plugin_name = gst_plugin_get_name (plugin);
+
+  /* not interested in typefind factories, only element factories */
+  features = gst_registry_get_feature_list (gst_registry_get_default (),
+      GST_TYPE_ELEMENT_FACTORY);
+
+  for (l = features; l != NULL; l = l->next) {
+    GstPluginFeature *feature;
+
+    feature = GST_PLUGIN_FEATURE (l->data);
+
+    /* only interested in the ones that are in the plugin we just loaded */
+    if (g_str_equal (plugin_name, feature->plugin_name)) {
+      GstElementFactory *factory;
+
+      g_print ("element-%s\n", gst_plugin_feature_get_name (feature));
+
+      factory = GST_ELEMENT_FACTORY (feature);
+      print_plugin_automatic_install_info_protocols (factory);
+      print_plugin_automatic_install_info_codecs (factory);
+    }
+  }
+
+  g_list_foreach (features, (GFunc) gst_object_unref, NULL);
+  g_list_free (features);
+}
+
 int
 main (int argc, char *argv[])
 {
   gboolean print_all = FALSE;
+  gboolean print_aii = FALSE;
   GOptionEntry options[] = {
     {"print-all", 'a', 0, G_OPTION_ARG_NONE, &print_all,
         N_("Print all elements"), NULL},
+    {"print-plugin-auto-install-info", '\0', 0, G_OPTION_ARG_NONE, &print_aii,
+        N_("Print a machine-parsable list of features the specified plugin "
+              "provides.\n                                       "
+              "Useful in connection with external automatic plugin "
+              "installation mechanisms"), NULL},
     GST_TOOLS_GOPTION_VERSION,
     {NULL}
   };
@@ -1108,6 +1253,9 @@ main (int argc, char *argv[])
   textdomain (GETTEXT_PACKAGE);
 #endif
 
+  if (!g_thread_supported ())
+    g_thread_init (NULL);
+
   ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
@@ -1149,8 +1297,12 @@ main (int argc, char *argv[])
 
       /* if there is such a plugin, print out info */
       if (plugin) {
-        print_plugin_info (plugin);
-        print_plugin_features (plugin);
+        if (print_aii) {
+          print_plugin_automatic_install_info (plugin);
+        } else {
+          print_plugin_info (plugin);
+          print_plugin_features (plugin);
+        }
       } else {
         GError *error = NULL;
 
@@ -1158,15 +1310,19 @@ main (int argc, char *argv[])
           plugin = gst_plugin_load_file (arg, &error);
 
           if (plugin) {
-            print_plugin_info (plugin);
-            print_plugin_features (plugin);
+            if (print_aii) {
+              print_plugin_automatic_install_info (plugin);
+            } else {
+              print_plugin_info (plugin);
+              print_plugin_features (plugin);
+            }
           } else {
-            g_print ("Error loading plugin file: %s\n", error->message);
+            g_print (_("Could not load plugin file: %s\n"), error->message);
             g_error_free (error);
             return -1;
           }
         } else {
-          g_print ("No such element or plugin '%s'\n", arg);
+          g_print (_("No such element or plugin '%s'\n"), arg);
           return -1;
         }
       }
